#include <cassert>
#include "Game/ConvBubble.h"
#include "Game/OSD.h"

namespace Salp {
namespace Game {

ConvBubble::ConvBubble(const Tile::Sequence *tseq,
               const char * bubtext, const Uint32 id) : Item(tseq, id)
{
    setType(getType()  & ~GI_CAN_MOVE & ~GI_CAN_COLLIDE_WITH_ITEM);
    _framesleft = 30 * 10;
    _bubtext = bubtext;
}

ConvBubble::~ConvBubble()
{
}

void ConvBubble::animate()
{
    if(!--_framesleft)
        getGameState()->delItemDelayed(this);
}

void ConvBubble::drawImage(SDL_Surface *screen, const Sint32 topx, const Sint32 topy)
{

    // don't bother drawing it if it's off the screen
    if(_xpos >= topx +  screen->w ||
            _ypos >= topy + screen->h ||
            (Sint32) (_xpos + _width) <= topx ||
            (Sint32) (_ypos + _height) <= topy)
        return;
    
    // Do a normal draw
    Item::drawImage(screen, topx, topy);
    
    // Draw text
    int cwidth  = (_width - 4) / 6; // (width - padding) / (char_width +spacing)
    int cheight = (_height - 10) / 6; // (width - padding) / (char_width +spacing)
    int col = 0;
    int row = 0;
    int lenleft = _bubtext.length();
    while(row < cheight && lenleft > 0 && screen->h > _ypos - topy + 5 + row * 6) {
        OSD::drawText5x5(screen, _xpos - topx + 2, _ypos - topy + 5 + row * 6,
                         (char *) _bubtext.c_str() + col, lenleft>cwidth?cwidth:lenleft);
        ++row;
        col += cwidth;
        lenleft -= cwidth;
    }
}

} // namespace Game
} // namespace Salp
