#include <assert.h>
#include "Game/ItemPlatform.h"

#include <time.h>

namespace Salp {
namespace Game {

ItemPlatform::ItemPlatform(const Tile::Sequence *tseq,
               const Uint32 id) : Item(tseq, id)
{
    _movespeed = 1;   // one px at a time please

    _path = 0;         // no path yet

    setType(
             getType()// 
   //       & ~GI_USE_VEL_THRESH           // UNUSED
   //       &  ~GI_ON_COLLIDE_LOSE_VEL 
   //       &  ~GI_ON_COLLIDE_GIVE_VEL 
   //       &  ~GI_ON_COLLIDE_TAKE_VEL 
            & ~GI_IS_AFFECTED_BY_GRAVITY 
   //       &  ~GI_IS_AFFECTED_BY_TRACTION 
   //       &  ~GI_CAN_RIDE_ITEMS
        );
        
        _bounce_seq = 0;
        _default_seq = (Tile::Sequence* )tseq;
        _bounce_dir = 0;
}

ItemPlatform::~ItemPlatform()
{
    delete _path;
}

void ItemPlatform::animate()
{
    static int targx, targy;

    assert(_path != 0);
    _path->getTargCoord(targx, targy);

    // if we reached the target point, then set
    // the target to the next control point.
    if(getXPos() == targx && getYPos() == targy) {
        _path->nextCoord();   
        _path->getTargCoord(targx, targy);
    }
    
    setVelTowardCoord(targx, targy, _movespeed);
}

Path * ItemPlatform::getPath() const
{
    return _path;
}

unsigned int ItemPlatform::getSpeed() const
{
    return _movespeed;
}

void ItemPlatform::onCollisionItem(Item *victim, unsigned int vic_collide_status)
{

      // If we have a bounce_dir, and the victim collided in one
      // of those directions, we want to have fun with them.
      if(vic_collide_status & _bounce_dir) {
          victim->setYVel(-20);
          
          // make pretty animation if we can
          if (_bounce_seq)
              setSequenceHard(_bounce_seq);
              //createEffect(_bounce_effect, _bounce_dir);
      }
}
 
void ItemPlatform::onCollisionMap()
{
    
}

void ItemPlatform::onDrewLastFrame()
{
    if (_bounce_seq && getSequence() == _bounce_seq)
        setSequence(_default_seq);
}
 

void ItemPlatform::setBounceEffect(Tile::Sequence *tseq, int dir)
{
    assert(tseq != 0);
    assert(dir & DIR_ANYWHERE);
    
    _bounce_seq = tseq;
    _bounce_dir = dir;
    
    return;
}

void ItemPlatform::setPath(Path *path)
{
    _path = path;
}

void ItemPlatform::setSpeed(unsigned int speed)
{
     _movespeed = speed;
}

} // namespace Game
} // namespace Salp
