#include <cassert>
#include "Game/ItemPowerUp.h"
#include "Game/Player.h"
#include "Game/Event.h"
#include "General/media.h"

namespace Salp {
namespace Game {

ItemPowerUp::ItemPowerUp(const Tile::Sequence *tseq,
               const Uint32 id) : Item(tseq, id)
{

    setType(
             getType()
   & ~GI_IS_AFFECTED_BY_GRAVITY 
   & ~GI_ON_COLLIDE_REPOS
        );
    
    _wstat = 0;        
}

ItemPowerUp::~ItemPowerUp() {
    delete _wstat;
}

void ItemPowerUp::onCollisionMap()
{

}

void ItemPowerUp::onCollisionItem(Item *victim, unsigned int vic_collide_status)
{
    assert(victim != 0);
    // We collided with somebody, yippie!
    // Let's give'em what we've got.
    
    if (victim->isPlayer()) {
        if (_wstat)
            ((Game::Player *) victim)->addAmmo(_wstat);
        if (getHP() > 0)        victim->incHP(getHP());
        else if (getHP() < 0)   victim->decHP(-getHP());
    }
    
    createEffect(General::seq_powerup_taken);
    
    // Then it's time for us to disappear.
    Event e = Game::Event(time(0) + 30);
    e.setTypeAddItem(this);
    getGameState()->addEvent(e);

    // only *remove* the item from game state; I still exist
    // in memory somewhere
    getGameState()->delItem(getId(), true); 
    
}

void ItemPowerUp::onOOB(bool x_oob, bool y_oob) {
    getGameState()->delItemDelayed(this);
}


} // namespace Game
} // namespace Salp
