#include <assert.h>
#include <cstdlib>
#include "Game/NPC.h"
#include "Game/ItemProjectile.h"
#include "Game/Scoreboard.h"
#include "Game/OSD.h"
#include "General/general.h"
#include "General/media.h"
#include <cstring>

using namespace Salp::General;

namespace Salp {
namespace Game {

vector <string *> NPC::_used_names;

Game::Sound    *NPC::__snd_fire = 0;
Tile::Sequence *NPC::__seq_proj_tracer = 0;
Tile::Sequence *NPC::__seq_proj_homing = 0;

NPC::NPC(const Tile::Sequence *tseq,
               const Uint32 id) : Item(tseq, id)
{
    // initialize key states (set all to 0)
    setHP(100); // default hp
    setMaxHP(100);
    _has_unique_name = false;
    _target = 0;
    setType(getType() & ~GI_USE_LOOSE_OOB);
}

NPC::~NPC()
{
    // find name entry in unique table and delete it
    // from the table (remember, they're just pointers ..
    // and we dont want to delete what they point to)
#warning Not sure if i looked this code over yet..
    if (_has_unique_name) {
        vector<string *>::iterator i;
        for (i=_used_names.begin(); i != _used_names.end(); ++i) {
            if (*i == &_name) {
                _used_names.erase(i);
                break;
            }
        }
    }
}

void NPC::appendConsoleMsg(string msg)
{
    assert(_pdata != 0);
    
    if (_pdata != 0) {
        unsigned short msg_len = 4 + msg.length();
        char *data = new char[msg_len];
        SET_SMSG_LEN(data, msg_len);
        SET_SMSG_TYPE(data, MS_CONSOLE_MESSAGE);
        strncpy(data + 4, msg.c_str(), msg.length());
        _pdata->sendMsg(data, msg_len);

        delete[] data;
    }
}

void NPC::drawName(SDL_Surface *screen, const Sint32 topx, const Sint32 topy)
{
    static Uint32   msgbkgdcolor  = SDL_MapRGB(screen->format, 51, 51, 51);
    static SDL_Rect dstrect;
    dstrect.w = _name.length() * 6 - 1;
    dstrect.x = getXPosC() - topx - (dstrect.w >> 1) + 1;
    dstrect.y = getYPos() - topy + getHeight() - 13;
    dstrect.h = 5;
    
    assert(canDrawImage(screen, topx, topy));
    
    General::fillRectAlpha(screen, &dstrect, msgbkgdcolor, 128);

   
    OSD::drawText5x5(screen,
                     dstrect.x,  // X
                     dstrect.y,                         
                     (char *) _name.c_str(),
                     _name.length());
}


void NPC::fireHoming(Game::NPC *target)
{
    assert(target->isNPC());
    
    Game::ItemProjectile *iproj = new Game::ItemProjectile(__seq_proj_homing, WEAPON_HOMING);
    iproj->setHP(5);
    iproj->setPosC(this);
    iproj->setMap(getMap());
    iproj->setTeam(getTeam());

    iproj->setOwner(this);
    iproj->setTarget(target);
    iproj->setSpeed(8);
    getGameState()->addItem(iproj);
}


void NPC::fireRocket()
{
    Game::ItemProjectile *iproj = new Game::ItemProjectile(
                                      getGameState()->getTileSequence("tiles/RocketThin/1"),
                                      WEAPON_ROCKET);

    iproj->setHP(20);
    iproj->setPosC( this );
    iproj->setMap(getMap());
    iproj->setYVel(-5);
    iproj->setTeam(getTeam());
    iproj->setOwner(this);
    getGameState()->addItem(iproj);
}

void NPC::fireSalp(const bool primary)
{

    Game::ItemProjectile *iproj = new Game::ItemProjectile(
        seq_salp_vomit[isFacingRight()?8:6],
                                      WEAPON_SALP);
    iproj->setHP(10);
    iproj->setTypeOn(GI_IS_AFFECTED_BY_GRAVITY);
    iproj->setPos( getXPos() +  (getType() & GI_FACING_RIGHT?iproj->getWidth():0), getYPos() + 12);
    iproj->setMap(getMap());
    
    if (primary) {
        // frog/grape
        iproj->setYVel(getYVel() - (rand() % 6));
        if (rand() % 2) {
            iproj->setSequence(seq_salp_grape);
            iproj->setCollideEffectDirectional(&seq_salp_grape_explosion[0]);
        } else {
            iproj->setSequence(seq_salp_frog);
            if (isFacingRight())  iproj->faceRight();
        }
    } else {
        __snd_fire->playOnce();
        iproj->setCollideEffectDirectional(&seq_salp_vomit_explosion[0]);
        iproj->setCollideSound(General::sound_salp_splat);
    }

    iproj->setXVel((isFacingRight()?1:-1) * (abs(getXVel() << 1) + 5));
    iproj->setTeam(getTeam());

    iproj->setOwner(this);
    getGameState()->addItem(iproj);
}

void NPC::fireSpread(const bool *keys)
{
    Game::ItemProjectile *iproj;

    Sint32 cx, cy;
    getPosC(cx, cy);
        
    int base_xvel = (isFacingRight()?1:-1) * (abs(getXVel()) + 10);
    int base_yvel = abs(base_xvel);
    int base_pos;   // first position, measured in pi/4's, for shot to follow path of. moves counter-clockwise.
    
    if (keys[KEY_UP]) 
        base_pos = (keys[KEY_RIGHT] || keys[KEY_LEFT]) ? 1 : 3; 
    else if (keys[KEY_DOWN])
        base_pos = (keys[KEY_RIGHT] || keys[KEY_LEFT]) ? -3 : -5; 
    else
        base_pos = -1;
        
    for(int i=0,pos=base_pos; i<3; ++i,pos++) {
        iproj = new Game::ItemProjectile( seq_spreadshot_bullets,
                                          WEAPON_SPREAD);
        iproj->setHP(5);
        iproj->setPos( cx - iproj->getWidth()/2,
                       cy - iproj->getHeight()/2 );
        iproj->setMap(getMap());
        
//        iproj->setXVel(int(cos((i/float(pieces)-.5) * 3.14))* base_xvel);
//        iproj->setYVel(int(sin((i/float(pieces)-.5) * 3.14) * 10));
        iproj->setXVel( int(cos(pos* -3.14/8) * base_xvel));
        iproj->setYVel( int(sin(pos* -3.14/8) * base_yvel));
        iproj->setTeam(getTeam());
    iproj->setOwner(this);

        getGameState()->addItem(iproj);
    }

}

void NPC::fireShotgun(const bool *keys)
{
    General::sound_shotgun->playOnce();

    Game::ItemProjectile *iproj = new Game::ItemProjectile(
                                      seq_shotgun_bullets[0],
                                      WEAPON_SHOTGUN);
    iproj->setHP(10);
    iproj->setTypeOff(GI_IS_AFFECTED_BY_GRAVITY);
//    iproj->setPos( getXPos() +  (getType() & GI_FACING_RIGHT?iproj->getWidth():0), getYPos() + 12);
    iproj->setPosC( this);
    iproj->setYPos(iproj->getYPos() - 10);
    iproj->setMap(getMap());
    
    int base_pos;
    if (keys[KEY_UP])        base_pos = (keys[KEY_RIGHT] || keys[KEY_LEFT]) ? 2: 4; 
    else if (keys[KEY_DOWN]) base_pos = (keys[KEY_RIGHT] || keys[KEY_LEFT]) ?-2:-4; 
    else                     base_pos = 0;

    int base_vel = isFacingRight()?20:-20; // should not be more than 32.. even so, may not collide using current collision detection when the victim is near the wall (unless server renders ALL frames of animation separately!)

    iproj->setXVel(  int(cos(base_pos* 3.14/8) * base_vel));
    iproj->setYVel( -int(sin(base_pos* 3.14/8) * 20));
    
    iproj->setSequence(seq_shotgun_bullets[ sign(iproj->getXVel()) + 1 + (sign(iproj->getYVel())+1)*3  ]);
    
    iproj->setTeam(getTeam());

    iproj->setOwner(this);
    getGameState()->addItem(iproj);
}

void NPC::fireTracer()
{
    //__snd_fire->playOnce();

    Game::ItemProjectile *iproj = new Game::ItemProjectile(__seq_proj_tracer, WEAPON_TRACER);
    iproj->setHP(1);
//    iproj->setTypeOff(GI_IS_AFFECTED_BY_GRAVITY);
    iproj->setPos( getXPos() +  (getType() & GI_FACING_RIGHT?iproj->getWidth():0),
                   getYPos() + 12);
    iproj->setMap(getMap());
    iproj->setXVel((getType() & GI_FACING_RIGHT?1:-1) * (abs(getXVel() << 1) + 5));
    iproj->setTeam(getTeam());

    iproj->setOwner(this);
    iproj->setOwnerTargetOnHit(true);
    getGameState()->addItem(iproj);
}

void NPC::fireWeapon(const WeaponType t, const bool primary, const bool * keys) 
{
    switch (t) {
        case WEAPON_HOMING: 
            if (primary) {
                if (_target && _target->isNPC())  fireHoming((NPC *) _target);
                else if (useTrainingMode) appendConsoleMsg("* Hit someone with a tracer first.");
                else fireTracer();
            } else fireTracer();
            break;
        case WEAPON_SALP:
            fireSalp(primary);
            break;
        case WEAPON_SPREAD:
            fireSpread(keys);
            break;
        case WEAPON_SHOTGUN:
            fireShotgun(keys);
            break;
        default:
            assert(0); // unsupported weapon
    }
}


Game::Gender & NPC::getGender() 
{
    return _gender;
}

int NPC::getMaxHP()
{
    return _maxhp;
}

bool NPC::isNameUnique(const string& name)
{
    // Returns true if name is NOT found in our
    // list of used names. Else returns false.
//    cout << "Checking " << name << " vs:\n";
    for (unsigned int i=0; i<_used_names.size(); ++i) {
//        cout << "    o " << *_used_names[i] << endl;
        if (strcasecmp(_used_names[i]->c_str(), name.c_str()) == 0)
            return false;
    }
    
    return true;
}

void NPC::onHit(Game::Item *victim, Game::ItemProjectile *iproj) 
{
}

void NPC::onOOB(bool x_oob, bool y_oob)
{
    setPosClipToMap();
}

void NPC::setMaxHP(int maxhp)
{
    _maxhp = maxhp;
}

void NPC::setNameUnique(string name)
{
    assert(_name != name);             // we won't ever set it to the same value.
    assert(_name == "" && name != ""   ||
           _has_unique_name
          ); // code cannot be replayed usually
    
    if (!isNameUnique(name)) {
        int addval = 0;
        char tmpstr[18];
        do {
            ++addval;
            sprintf(tmpstr, "%d", addval);
        } while (!isNameUnique(name + tmpstr));
        name += tmpstr;
    }
    
    setName(name);
    
    // Don't register the name more than once
    if (!_has_unique_name) {
        _used_names.push_back(&_name);
        _has_unique_name = true;
    }
}

void NPC::setTarget(Game::NPC *target)
{
    // sets my target to point to an NPC
    _target = target;
}

} // namespace Game
} // namespace Salp
