#ifndef __GAMENPC_H
#define __GAMENPC_H

#include "SALPSTUB.h"
#include "Game/Item.h"
#include "Game/Gender.h"
#include <vector>

namespace Salp {
namespace Game {
class PeerData;

class NPC : public Item
{
    public:
        explicit NPC(const Tile::Sequence *tseq,
                        const Uint32 id=0);
        virtual ~NPC();
        void appendConsoleMsg(string msg);
        void drawName(SDL_Surface *screen, const Sint32 topx, const Sint32 topy);
        void fireHoming(Game::NPC *target);
        void fireRocket();
        void fireSalp(const bool upward);
        void fireShotgun(const bool *keys);
        void fireSpread(const bool *keys);
        void fireTracer();
        void fireWeapon(const WeaponType t, const bool primary, const bool *keys);
        
        Game::Gender & getGender();
        unsigned int getItemType() const { return ITYPE_NPC; };
        int getMaxHP();
        string getName() { return _name; }
        
        virtual void onHit(Game::Item *victim, Game::ItemProjectile *iproj); // If you derive from this, make sure you call NPC's!
        void setMaxHP(int maxhp);
        void setName(const string name) { _name = name; setDirty(DIRTY_NAME); }
        void setNameUnique(const string name);
        void onOOB(bool x_oob, bool y_oob);
        void setTarget(Game::NPC *target);
        void setPeerData(PeerData * pdata) { _pdata = pdata; }

        static Game::Sound    *NPC::__snd_fire;
        static Tile::Sequence *NPC::__seq_proj_tracer;
        static Tile::Sequence *NPC::__seq_proj_homing;
        
        friend class Salp::Game::Item;
        
    private:
        static vector <string *> _used_names;
        static bool isNameUnique(const string& name);
        
    protected:
        int _maxhp;
        string _name;
        bool _has_unique_name;
        Game::Gender _gender;
        Game::Item * _target;
        PeerData *_pdata;

};

} // namespace Game
} // namespace Salp

#endif
