#ifndef __GAME_BUSHBOMBER_H
#define __GAME_BUSHBOMBER_H

#include "NPCBushBomber.h"
#include "ItemProjectile.h"

namespace Salp {
namespace Game {

Tile::Sequence *NPCBushBomber::__seq_peek = 0;
Tile::Sequence *NPCBushBomber::__seq_walk = 0;
Tile::Sequence *NPCBushBomber::__seq_idle = 0;

NPCBushBomber::NPCBushBomber(const Tile::Sequence *tseq, 
                             const Uint32 id) : NPC(tseq, id)
{
    _bushstate = BUSH_IDLING; // what is the bush doing right now?
}

void NPCBushBomber::animate() {
    updateSequence();
}

Game::Item * NPCBushBomber::getTarget() {
    return _target;
}

void NPCBushBomber::onUpdateTarget() {
    updateSequence();
}
void NPCBushBomber::onDrewLastFrame() {
    if (getSequence() == __seq_peek) {
    //    cout << "SET IDLE\n";
        setSequence(__seq_idle);
        fireProjectile();
    }
}

void NPCBushBomber::setTarget(Game::Item *target) {
    if (target != _target) {
        _target = target;
        onUpdateTarget();
    }
}

void NPCBushBomber::updateSequence() {
    if(_target) {
        if (abs(getXPos() - _target->getXPos()) < getWidth()*2) {
            // attack!
            setXVel(0);
            setSequence(__seq_peek);
        } else {
            // sneak up
            setSequence(__seq_walk);
            setXVel(_target->getXPos() > getXPos() ? 1 : -1);
            if(getXVel() < 0) faceLeft(); else faceRight();
        }
    } else {
        setSequence(__seq_idle);
    }
}

/*
1: NPC
2: target

x1 + t*xv1 = x2 + t*xv2

x1 - x2 + t * ( xv1 - xv2 ) = 0
y1 - y2 + t * ( C - xv1 - yv2 ) = 0


x1 - x2 + t * ( xv1 - xv2 ) = y1 - y2 + t * ( C - xv1 - yv2 )
x1/t - x2/t + xv1 - xv2 = y1/t - y2/t + C - xv1 - yv2
x1/t - x2/t - y1/t + y2/t + xv1 - xv2 = C - xv1 - yv2
(x1 - x2 - y1 + y2)/t + xv1 - xv2 = C - xv1 - yv2
(x1 - x2 - y1 + y2)/t - xv2 = C - 2*xv1 - yv2
((x1 - x2 - y1 + y2)/t - xv2 - C + yv2)/2 = xv1


C is some constant. Let's assume 1 for fun.
xv1 + yv1 = C
yv1 = C - xv1

*/

void NPCBushBomber::fireProjectile()
{
 //   __snd_fire->playOnce();
    Game::ItemProjectile *iproj = new Game::ItemProjectile(
                                      getGameState()->getTileSequence("tiles/Starshot/2"), WEAPON_UNKNOWN);

    iproj->setPos( getXPos() +  (getType() & GI_FACING_RIGHT?iproj->getWidth():0),
                   getYPos() + 12);
    iproj->setMap(getMap());
    iproj->setXVel((getType() & GI_FACING_RIGHT?1:-1) * (abs(getXVel()) + 5));
    iproj->setTeam(getTeam());
    iproj->setOwner(this);
    getGameState()->addItem(iproj);
}

} // namespace Game
} // namespace Salp

#endif
