#ifndef __OSD_H
#define __OSD_H

#include "SALP_SDL.h"
#include <deque>
#include "SALPSTUB.h"
#include "Tile/Set.h"
#include "Game/Player.h"

namespace Salp {
namespace Game {

// Class for SW's On-Screen Display
//

    class ConsoleMsg {
      public:

        ConsoleMsg() {
            time_sent = SDL_GetTicks();
        };
        
        string        msg;
        Uint32 time_sent;
    };

class OSD
{
    public:
        OSD(Tile::Set *score, Tile::Set *health, Tile::Set *alpha, Game::State *gs, SDL_Surface *screen);
        ~OSD();
        
        void animate();
        bool appendCmdLine(const unsigned int sym); // appends msg to command line
        void appendMsg(string msg); // Adds message to console.
        static void drawText5x5(SDL_Surface *screen, int x, int y, char * text, int len=-1);
        void drawConsoleMsgs(SDL_Surface *screen);
        void drawImage(SDL_Surface *screen);
        void drawRadar(SDL_Surface *screen);
        void drawScore(SDL_Surface *screen, int score, int x, int y, int alpha=0);
        void expireOldMsgs();
        bool getShowRadar() { return _show_radar; };
        void setCmdLine(const string& cmd);
        void setEchoSTDOUT(bool echo) { _echo_stdout = echo; }
        void setPlayer(Game::Player *player);
        void setShowRadar(bool show_radar) { _show_radar = show_radar; }
    private:
        static Tile::Set *_score_tiles, 
                         *_health_tiles,
                         *_alpha_tiles;
        Game::State  * _gs;
        Game::Player *_player; // Pointer to player associated with OSD
        PlayerRec * _prec;
        int _score;              // could be negative i guess
        deque<ConsoleMsg *> _conmsgs; // console messages
        string _cmdline;
        bool _echo_stdout;
        bool _show_radar;
        SDL_Surface * _screen;
};


} // namespace Game
} // namespace Salp

#endif
