#ifndef __GAMEPLAYER_H
#define __GAMEPLAYER_H

#include "SALPSTUB.h"
#include "Game/NPC.h"
#include "Game/WeaponStatus.h"

namespace Salp {
namespace Game {

class Item;

#define MAX_KEYS 10


class Player : public NPC
{
    public:
        explicit Player(const Tile::Sequence *tseq,
                        const Uint32 id=0);
        virtual ~Player();
        void addAmmo(WeaponStatus *ws);
        void addAmmo(WeaponType t, int ammo, bool has_weapon=false);
        bool canFireCurrentWeapon();
        void doCmd(string cmd);
        void drawImage(SDL_Surface *screen,
                           const Sint32 topx,
                           const Sint32 topy);
        void fireWeaponCurrent(const bool primary);
        unsigned int getItemType() const { return ITYPE_PLAYER; };
        bool getKeyState(const unsigned int& keyId) const;
        virtual void setKeyState(const unsigned int& keyId, const bool& isDown);
        bool selectNextWeapon(bool reverse=false);
        
    private:
    
    protected:
        bool _keys[MAX_KEYS];
        int _hp, _maxhp;
        vector<WeaponStatus *> _weapons;
        unsigned int _active_weapon;
};

} // namespace Game
} // namespace Salp

#endif


/*
  - location-specific weapons, e.g. be by pipe to send salp through it
  - building anti-fire sprinklers shooting out salp??
*/
/*
    Level gimmicks
    --------------
       Goals
       ------
       
       - capture the flag
       - get to the top (reach finish line; race)
       - deathmatch
       - build robot/NPC from collecting parts (ala Rock recipes); 
           - can only hold onto one ingredient at a time, in 'monster room'
           - player-controlled robot (remote-control), or tank
       - people in two separate rooms, and each (control) room controls the traps in
         the other 
       
       Traps
       ------
       
       Giant flame thrower (activate by hitting button on wall in control room; DPAD-UP)
       Bowser-style lava.
       Floor-saw
       multi-windowed control room screen (640x480 mode?) to handle triggers
       
       
*/
