#include <assert.h>
#include "Game/PlayerStomper.h"
#include "Game/ItemProjectile.h"
#include "Game/ConvBubble.h"
#include "Game/NPCBushBomber.h"
#include "Game/ItemEffect.h"
#include "Game/State.h"
#include "Game/OSD.h"
#include "General/media.h"

namespace Salp {
namespace Game {

// Static member data declarations
Tile::Sequence *PlayerStomper::__seq_jump = 0;
Tile::Sequence *PlayerStomper::__seq_walk = 0;
Tile::Sequence *PlayerStomper::__seq_idle = 0;
Tile::Sequence *PlayerStomper::__seq_run  = 0;
Tile::Sequence *PlayerStomper::__seq_poofin  = 0;
Tile::Sequence *PlayerStomper::__seq_poofout  = 0;
Game::Sound    *PlayerStomper::__snd_jump = 0;
Game::Sound    *PlayerStomper::__snd_land = 0;

PlayerStomper::PlayerStomper(const Tile::Sequence *tseq,
                             const Uint32 id) :
        Player((PlayerStomper::__seq_idle?PlayerStomper::__seq_idle:tseq), id)
{
    _jframesleft = 0;
    setXVelThresh(10);
    assert( __seq_jump != 0 );
    assert( __seq_walk != 0 );
    assert( __seq_idle != 0 );
    assert( __seq_run != 0);
    assert( __seq_poofin != 0);
    assert( __seq_poofout != 0);
    assert( __seq_proj_tracer != 0);
    assert( __seq_proj_homing != 0);


    WeaponStatus *ws;
    
    ws = new WeaponStatus(WEAPON_SALP , true);
    assert(ws != 0);
    ws->ammo_count = 20;
    _weapons.push_back(ws);

/*
    ws = new WeaponStatus(WEAPON_SHOTGUN, true);
    assert(ws != 0);
    ws->ammo_count = 20;
    _weapons.push_back(ws);

    ws = new WeaponStatus(WEAPON_SPREAD, true);
    assert(ws != 0);
    ws->ammo_count = 20;
    _weapons.push_back(ws);
    
    ws = new WeaponStatus(WEAPON_HOMING, true);
    assert(ws != 0);
    ws->ammo_count = 20;
    _weapons.push_back(ws);
*/
}

void PlayerStomper::animate()
{
    // Use player key data to determine velocity
    int dxv = (_keys[KEY_LEFT]?-1:0 +
               _keys[KEY_RIGHT]?1:0);

    if(isGrounded() && dxv) {
        dxv <<= 1; // 2 or 0
        int currxvel = getXVel();

        if( _keys[KEY_L] &&
                (currxvel > 15 || currxvel < 15)
          )
            dxv += dxv>0?5:-5; // dxv *= 2
        else if( !_keys[KEY_L] &&
                 (currxvel > 5 || currxvel < -5))
            dxv = 0;
        setXVel(currxvel + dxv);
    } else {
        if(abs(getXVel()) < 2) { setXVel(getXVel() + dxv*2); }
        else { setXVel(getXVel() + dxv); }
    }


/*    if (dxv) {
        if (_keys[KEY_L]) setXVel(dxv *  10);
        else setXVel(dxv *  5);
    } else 
        setXVel(0);
*/
    // add cheap x friction
    if(getXVel() < 0) setXVel(getXVel() + 1);
    else if(getXVel() > 0) setXVel(getXVel() - 1);

    //OLDOLDOLD    setXVelThresh( _keys[KEY_B] && isRunning()?8:4 );

    // If I'm jumping
    if(_jframesleft) {
        // move up farther!
        setYVel(getYVel() - 1);
        // dec a frame
        --_jframesleft;
        updateSequence();
    }

}

void fireBushBomber()
{
/*    Game::NPCBushBomber *bush = new Game::NPCBushBomber();
    bush->setPos( getXPos() +  (getType() & GI_FACING_RIGHT?bush->getWidth():0),
                   getYPos() + 12);
    bush->setMap(getMap());
    bush->setTarget(this);
    getGameState()->addItem(bush);
return;
*/
}



void PlayerStomper::explode()
{
    
    /* CONVERSATION BUBBLE:
    Game::ConvBubble *bub = new Game::ConvBubble(getGameState()->getTileSequence("tiles/bubble76x48/1"), "Wazzzzup!");

    bub->setPosC(this);
    bub->setMap(getMap());    
    bub->setTeam(getTeam());
    getGameState()->addItem(bub);
    return;
    */
    
    Game::ItemProjectile *iproj;

    Sint32 cx, cy;
    getPosC(cx, cy);
        
    for(int i=0,pieces=20; i<pieces; ++i) {
        iproj = new Game::ItemProjectile( getGameState()->getTileSequence("tiles/Starshot/2"), WEAPON_UNKNOWN);

        iproj->setPos( cx - iproj->getWidth()/2,
                       cy - iproj->getHeight()/2 );
        iproj->setMap(getMap());
        
        iproj->setXVel(int(cos(i/float(pieces) * 6.28) * 10));
        iproj->setYVel(int(sin(i/float(pieces) * 6.28) * 10));
        iproj->setTeam(getTeam());
    iproj->setOwner(this);
        getGameState()->addItem(iproj);
    }
}

void PlayerStomper::setKeyState(const unsigned int& keyId, const bool& isDown)
{
    // do normal logging
    Player::setKeyState(keyId, isDown);

    // if they want to jump, and they're on the ground, let them
    if(keyId == KEY_B) {
        if(isDown && isGrounded()) {
//            __snd_jump->playOnce();
            General::sound_taer_jump.getItem()->playOnce();

            // If I've been running, set my initial jump to be higher.
            if(_keys[KEY_L] && abs(getXVel()) >= 10)
                setYVel(-10);
            else
                setYVel(-8); // -10
            _jframesleft = 16;
            updateSequence();
        } else {
            _jframesleft = 0;
        }
    } else if (isDown && keyId == KEY_Y) {
        fireWeaponCurrent(true);
    } else if (isDown && keyId == KEY_X) {
        fireWeaponCurrent(false);
    } else if (isDown && keyId == KEY_A) {
        if (selectNextWeapon()) {
            appendConsoleMsg("Selected " + _weapons[_active_weapon]->getName() + ".");
        } else {
            appendConsoleMsg("No other weapons have ammo.");
        }
    } else if (isDown && keyId == KEY_R) {
        // tauntometer
        General::sound_taer_taunt.getItem()->playOnce();
    } else if (keyId & (KEY_LEFT | KEY_RIGHT)) {
        // or if they're moving, update the sequence
        updateSequence();
    }
}

void PlayerStomper::onCollisionItem(Item *victim, unsigned int vic_collide_status)
{
    if( (getStatus() & GI_FORCED_UP) &&
        (getStatusDiff() & GI_FORCED_UP) ) {
        __snd_land->playOnce();
    }
    updateSequence();
}

void PlayerStomper::onCollisionMap()
{
    if( (getStatus() & GI_FORCED_UP) &&
        (getStatusDiff() & GI_FORCED_UP) ) {
        __snd_land->playOnce();
    }
    
    updateSequence();
}

void PlayerStomper::onDeath(Item *killer)
{

    if(killer) {
        if (killer == this) {
            getGameState()->appendConsoleMsgAll(getName() + " killed " + getGender().getHis() + " own dumb self.");
        } else if (killer->isNPC()) {
            getGameState()->appendConsoleMsgAll( ((NPC *) killer)->getName() + " killed " + getName() + ". Yay " + ((NPC *) killer)->getGender().getHim());
        } else {
            getGameState()->appendConsoleMsgAll(getName() + " was mysteriously killed.");
        }
    } else {
        getGameState()->appendConsoleMsgAll(getName() + " was super-mysteriously killed.");
    } 

    // Make a noise
    General::sound_taer_ouch.getItem()->playOnce();

    // Set poofy cloud where he died
    Game::Item *deathcloud = new ItemEffect(__seq_poofout);
    deathcloud->setPosC(this);
    deathcloud->setMap(getMap());
    getGameState()->addItem(deathcloud);

    // Recycle player
    setHP(getMaxHP()); // Heal him

    setXVel(0); setYVel(0); // No more moveymove

    setPosStart(); // Go to a start position

    // Set poofy cloud where he reappears
    deathcloud = new ItemEffect(__seq_poofin);
    deathcloud->setPosC(this);
    deathcloud->setMap(getMap());
    getGameState()->addItem(deathcloud);
}

void PlayerStomper::onKill(Item *victim)
{

}

void PlayerStomper::updateSequence()
{
    if(!(getStatus() & GI_FORCED_UP) && _keys[KEY_B]) {
        setSequence(PlayerStomper::__seq_jump);
    } else if (isGrounded()) {
        if(getXVel() && canMoveInXVel()) {
            if(_keys[KEY_L])   setSequence(PlayerStomper::__seq_run); 
            else               setSequence(PlayerStomper::__seq_walk);
        } else { setSequence(PlayerStomper::__seq_idle); }
    } else {
        setSequence(PlayerStomper::__seq_idle);
    }
}

} // namespace Game
} // namespace Salp
