#include <iostream>
#include <cassert>

#include "Server.h"
#include "PlayerStomper.h"
#include "General/general.h"
#include "SDL_thread.h"
#include "Game/Scoreboard.h"
#include "General/graphics.h"

#include <iostream>
#include <iomanip>

using namespace std;
using namespace Salp::General;

// PURE EVIL!! (But hey, until standardize on UDP, why not :-))
// Let's hope they have the same flavor of SDL that we do!! :)
struct _TCPsocket {
        int ready;
        int channel; // was SOCKET, but is int for us
        IPaddress remoteAddress;
        IPaddress localAddress;
        int sflag;
};

namespace Salp {
namespace Game {

Game::Server * Server::_server = 0; // global access - this is the most recent server created


bool isMessageWellFormed(char *msg, int len) {
    // Checks message format to at least make sure the
    // LEN and TYPEs look half decent.
    char *mptr_targ = msg + len; // first non-message character
    int i =0;
    while (msg < mptr_targ) {
        assert(SMSG_LEN(msg) <= 1000);
        if (SMSG_LEN(msg) > 1000) return false; // eew 

        assert(SMSG_LEN(msg) >= 4);
        if (SMSG_LEN(msg) < 4) return false; // eew 
        
        assert(SMSG_TYPE(msg) != 0);
        if (SMSG_TYPE(msg) == 0) return false;
        
        assert(SMSG_TYPE(msg) < 100);
        if (SMSG_TYPE(msg) >= 100) return false;
        
        i   += SMSG_LEN(msg);
        msg += SMSG_LEN(msg); // move to next message
    }
    
    assert(msg == mptr_targ);
    return (msg == mptr_targ);
}


// #################################################
// ################### CLIENT ######################
// #################################################


Client::Client(Game::State *gs, Game::Player **gp) {
    assert(gs != 0);

    _gs         = gs;
    _mainplayer = gp;

    // allocate socket set for my one connection
    _set = SDLNet_AllocSocketSet(2);

    if (_set == NULL) {
        cerr << "Could not allocate socketset for client object (out of memory?).\n";
        exit(1);
    }

    // create peer data
    _peer = new PeerData;
    assert(_peer != 0);

    // then add it to our socket set.
    SDLNet_TCP_AddSocket(_set, _peer->tcpsock);
}

Client::~Client() {
    delete _peer;
    
}

bool Client::connectTo(string host, int port) {
    assert(_peer->tcpsock == 0);

    // resolve the hostname
    IPaddress ip;
    SDLNet_ResolveHost(&ip, (char *) host.c_str(), port);
    if(ip.host == INADDR_NONE) {
        cerr << "Could not resolve host " << host << ".\n";
        return false;
    }

    // Establish TCP connection
    _peer->tcpsock = SDLNet_TCP_Open(&ip);
    if (!_peer->tcpsock)
        return false;

    // ** LAME HACK **
    // This should be replaced with UDP code for the most part (I'd say..) by the time
    // the game goes production. Otherwise we really *WILL* have to require users to have
    // super-fast network connections (eewww)
    if (useTCPNoDelay) {
        int yes = 1;
        setsockopt(_peer->tcpsock->channel, IPPROTO_TCP, TCP_NODELAY, (char*)&yes, sizeof(yes));
    }

    // else we're there! hook it up into the sockset
    if(SDLNet_TCP_AddSocket(_set, _peer->tcpsock) == -1) {
        printf("SDLNet_AddSocket: %s\n", SDLNet_GetError());
        // perhaps you need to restart the set and make it bigger...
        exit(1);
    }    

    // --- Bind UDP port
    // ########## UDP CRAZINESS ##############
    _peer->udpport = 12468;
    _peer->udpsock = 0;
    while (!_peer->udpsock) 
        _peer->udpsock=SDLNet_UDP_Open(++_peer->udpport); // pick a port, any port
    
    if (!_peer->udpsock) {
        printf("Client SDLNet_UDP_Open: %s\n", SDLNet_GetError());
        exit(2);
    }

    ip.port++; // get next highest port for receivage
    _peer->udpchan=SDLNet_UDP_Bind(_peer->udpsock, -1, &ip);
    if (_peer->udpchan == -1) {
        printf("Client SDLNet_UDP_Bind: %s\n", SDLNet_GetError());
        exit(1);
    }

    if(SDLNet_UDP_AddSocket(_set, _peer->udpsock) == -1) {
        printf("SDLNet_AddSocket: %s\n", SDLNet_GetError());
        // perhaps you need to restart the set and make it bigger...
        exit(1);
    }   
    // ########## UDP CRAZINESS ##############

    return true;

}

void Client::handleServerMessages(char *msg, int len) {
     assert(isMessageWellFormed(msg, len));

     int n=0;
     
     while (n < len) {
         handleServerMessage(msg);
         n   += SMSG_LEN(msg);
         msg += SMSG_LEN(msg);
     }
}

void Client::handleServerMessage(char *msg) {
    // Note: this is like an if() { .. } right now,
    //       since we only have one event per readSock anyway.

    // got a message!
    if(debugProtocol)
        cout << "S   <" << setw(3) << SMSG_ELEN(msg) << ", " << setw(3) << SMSG_TYPE(msg) << ">: ";

    switch (SMSG_TYPE(msg)) {
    case MS_SET_PID: {
            // Server told me that I am player N. Set up the client so that
            // we view from player N's perspective.
            //
            assert(_mainplayer != 0);
            assert(*_mainplayer == 0);

            Uint32 pid;
            msg += 4; // get past header section
            DATA_IREAD_32(msg, pid);
 
            if (debugProtocol)
                cout << "You are player id " << pid << endl;

            *_mainplayer = (Game::Player *) _gs->getItem(pid);
            _gs->getOSD()->setPlayer(*_mainplayer);
            assert(*_mainplayer != 0);
            break;
        }

    case MS_LOAD_LEVEL:
        // Server says, "load this level!"
        if (debugProtocol)
            cout << "Load level " << SMSG_DATA(msg) << endl;
        _gs->loadLevel(SMSG_DATA(msg));
        break;

    case MS_MIN_VERSION: {
        assert(SMSG_ELEN(msg) == 2);

        Uint16 min_ver = ntoh16(*((Uint16 *) SMSG_DATA(msg)));

        // Server says, "this is the min version!"
        if (debugProtocol)
            cout << "I tolerate a min version of " << min_ver << endl;

        // Make sure we comply.
        if ( min_ver > SALP_WARS_VERSION ) {
            printf("#\n\n#\n# *** ERROR!!!!! ***\n#\n# Server supports min of version %05.2f; your version is %05.2f\n# Try downloading a newer version at http://salpwars.sourceforge.net/\n#\n#\n", float(min_ver)/100, float(SALP_WARS_VERSION)/100);
            exit(1);
        }

        // Otherwise tell the gamer just so he knows
        char vermsg[80];
        sprintf(vermsg, "## Server supports min version of %4.2f.", float(min_ver)/100);
        _gs->getOSD()->appendMsg(vermsg);

        break;
    }

    case MS_SOUND_PLAY_ONCE:

        if (debugProtocol)
            cout << "Play sound " << int(SMSG_DATA(msg)[0]) << endl;

        _gs->playSound(int(SMSG_DATA(msg)[0]));

        break;

    case MS_CREATE_ITEM: {
            Item *gi = Game::Item::createFromData(SMSG_DATA(msg));

            if (gi != 0) {
                if (debugProtocol)
                    cout << "Create item " << gi->getId() << endl;
                _gs->addItem(gi);
            } else {
                if (debugProtocol)
                    cout << "Create [?? existing ??] item " << ntoh32(*((Uint32 *) SMSG_DATA(msg))) << endl;
            }

            break;
        }

    case MS_UPDATE_ITEM: {
            if (debugProtocol)
                cout << "Update item data for " << ntoh32(*((Uint32 *) SMSG_DATA(msg))) << "\n";
            
            Item *gi = Game::Item::updateFromData(SMSG_DATA(msg));
            
            if (gi == 0 && debugProtocol)
                    cout << "\\--> ## ITEM DID NOT EXIST ##\n";

            break;
        }

    case MS_DELETE_ITEMS: {
            char *iptr = SMSG_DATA(msg);
            char *lasti = iptr + (SMSG_ELEN(msg) / sizeof(Uint32)) * sizeof(Uint32) ; // round down to nearest 4
            while(iptr < lasti) {
                if (debugProtocol)
                    cout << "Delete item " << ntoh32(*((Uint32 *) iptr)) << endl;

                _gs->delItem( ntoh32(*((Uint32 *) iptr)) );
                iptr += sizeof(Uint32);
            }

            break;
        }

    case MS_CONSOLE_MESSAGE: {
            if (debugProtocol)
                cout << "Append console message: '" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << "'\n";

            string newmsg = string(SMSG_DATA(msg), SMSG_ELEN(msg));
            _gs->getOSD()->appendMsg(newmsg);
            break; 
        }

    case MS_LOAD_TILE_SET: {
            if (debugProtocol)
                cout << "Load Tile Set: '" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << "'\n";
            
            assert(_gs != 0);
            Tile::Set *result = _gs->getTileSet(string(SMSG_DATA(msg), SMSG_ELEN(msg)), true);
            assert(result != 0);
            break; 
        }

    case MS_LOAD_SOUND: {
            if (debugProtocol)
                cout << "Load Sound: '" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << "'\n";
            
            assert(_gs != 0);
            Game::Sound *result = _gs->getSound(string(SMSG_DATA(msg), SMSG_ELEN(msg)), true);
            assert(result != 0);
            break; 
        }
        
    case MS_UPDATE_PLAYER_SCORE: {
        assert(_gs != 0);
        _gs->getScoreboard()->updatePlayerData(msg);
        break;
    }
    
    default:
        cout << "SERVER SENT UNKNOWN MESSAGE: len " << SMSG_LEN(msg)
        << ", type " << SMSG_TYPE(msg)
        << ", elen " << SMSG_ELEN(msg)
        << ", msg [" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << ']'
        << endl;
    };
}

bool Client::pollSocket() {
    // Polls peer socket and processes/input as possible.
    // Returns true if we found any activity.
    // Otherwise returns false.
    //
    
    assert(!_peer->hasError()); // no errors yet, right?

    // See how many sockets need stuff done to them
    int socks_ready = SDLNet_CheckSockets(_set, 1); // spend 1 msec

    // If none or error (eew), split.
    if(socks_ready <= 0)
        return false;

    // Otherwise let's see what we can do!
    if (SDLNet_SocketReady(_peer->udpsock)) {

        static UDPpacket **packets = SDLNet_AllocPacketV(4, 1024); // use max size here
	    if ( packets == 0 ) {
		    fprintf(stderr, "Couldn't allocate packets: Out of memory\n");
		    exit(2);
	    }

	    int n = SDLNet_UDP_RecvV(_peer->udpsock, packets);
	    while ( n-- > 0 ) {
//		    if ( packets[n]->channel >= 0 ) {
            handleServerMessages((char *) packets[n]->data, packets[n]->len);
//		    }
	    }
    } 

    if (SDLNet_SocketReady(_peer->tcpsock)) {
        if(!_peer->buf.readSock(_peer->tcpsock)) {
            // If we didn't get any data, we lost connection to the server!
            // Should probably handle this nicer.. :-)
            if (debugProtocol)
                cerr << "Client tried reading from socket but did not retrieve any data. Server probably disconnected.\n";
            cerr << "Lost connection to server.\n";

            exit(1);
        } else {
            char * msg;
            while((msg = _peer->buf.getEvent())) {
                // It's assumed here that msg's SMSG_LEN header is not lying
                handleServerMessage(msg);
                _peer->buf.resetBuf();
            }
        }
    }
    return true;
}


// #################################################
// ################### SERVER ######################
// #################################################

Server::Server(Game::State *gs) {
    assert(gs != 0);

    _bytes_mass_sent = 0;
    _gs              = gs;
    _listen          = 0;
    _udpsock         = 0;
    _udp_user_count  = 0;
    
    // allocate socket set for listener and sockets
    _set = SDLNet_AllocSocketSet(MAX_SOCKETS_SERVED + 1);

    if(_set == NULL) {
        cerr << "Could not allocate socketset for "
        << MAX_SOCKETS_SERVED << " sockets "
        << "(out of memory?)\n";

        exit(1);
    }

    _server = this;
}

Server::~Server() {
    // kill listener
    if(_listen != 0)
        SDLNet_TCP_Close(_listen);

    if(_udpsock != 0)
        SDLNet_UDP_Close(_udpsock);

    // close peer socks if applicable
    for(unsigned int peerid=0; peerid < _peers.size(); ++peerid) {
        if (_peers[peerid] != 0)
            delConnection(peerid, "Server object getting destroyed.");
    }

    if(_server == this)
        _server = 0;
}

unsigned int Server::getBytesMassSent() {
    return _bytes_mass_sent;
}

bool Server::listenOn(int port) {
    //
    // Set up our listener socket.
    //

    // set server ip
    IPaddress ip;
    SDLNet_ResolveHost(&ip, NULL, port);
    ip.host = INADDR_NONE; // cuz we're accepting socks

    // ----  Open TCP Port 
    _listen = SDLNet_TCP_Open(&ip);
    if (_listen == NULL)
        return false;

    // then add it to our socket set.
    SDLNet_TCP_AddSocket(_set, _listen);

    // ----  Open UDP Port
    // ########## UDP CRAZINESS ##############
    assert(!_udpsock);
    int max_try_ports = 1000; // try 1000 ports
    while (max_try_ports && !_udpsock)
         _udpsock = SDLNet_UDP_Open(port + 10 + max_try_ports--); // ..or .. open anonymous unused port (0)
         
    if(!_udpsock) {
        printf("SDLNet_UDP_Open: %s\n", SDLNet_GetError());
        return false;
    }
    // ########## UDP CRAZINESS ##############

    //
    // Yay, we're ready to take in players!
    //
    return true;
}

unsigned int Server::addConnection(TCPsocket new_sock) {
    // Adds connection to the list of peers, and returns
    // the connection's peerid.

    // create peer data
    PeerData *new_player = new PeerData;
    assert(new_player != 0);
    new_player->tcpsock = new_sock;

    // ** LAME HACK **
    // This should be replaced with UDP code for the most part (I'd say..) by the time
    // the game goes production. Otherwise we really *WILL* have to require users to have
    // super-fast network connections (eewww)
    if (useTCPNoDelay) {
        int yes = 1;
        setsockopt(new_sock->channel, IPPROTO_TCP, TCP_NODELAY, (char*)&yes, sizeof(yes));
    }

    //
    // Add new connection to our list of peers.
    //

    // find new slot in _peers (else append)
    unsigned int peerid=0;
    for (; peerid<_peers.size(); ++peerid) {
        if(_peers[peerid] == 0)
            break;
    }

    // If we *didn't* find empty slot, put new at end
    if(peerid == _peers.size())
        _peers.push_back(new_player);
    else
        _peers[peerid] = new_player; // else fill in

    new_player->peerid = peerid;

    if (debugProtocol)
        cout << "Adding socket for peer " << peerid << "\n";

    //
    // Add to select sock
    //
    SDLNet_TCP_AddSocket(_set, new_sock);


    // set up game state
    onConnect(new_player);


    // We got it! give them the sock's peerid
    return peerid;
}

void Server::delConnection(unsigned int peerid, string reason) {
    assert(peerid < _peers.size());
    assert(_peers[peerid] != 0);

    if (debugProtocol)
        cout << "Closing socket for peer " << peerid << ": " << reason << endl;
    
    PeerData *pdata = _peers[peerid];

    // clean up game state
    onDisconnect(pdata);

    // remove from socket set
    SDLNet_TCP_DelSocket(_set, pdata->tcpsock);
    
    // If they were using udp, dec our count
    if (pdata->udpsock)
        _udp_user_count--;

    // deallocate PeerData
    delete pdata; // closing of tcp/udp socks is done by ~PeerData

    // clean up peer list
    _peers[peerid] = 0;

}

Uint32 getPixel(SDL_Surface *image, unsigned short x, unsigned short y) {
    int bpp = image->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)image->pixels + y * image->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}

int Server::makeScreenshot(Game::State *gs, char *buf, int bufsize) {
    // Returns size of buf

    char *oldbuf = buf;

    // Get the screen
    SDL_Surface *screen = General::copyToTempSurface(gs->getScreen());

    assert(screen->w == 320);
    assert(screen->h == 240);
    assert(screen != 0);
    assert(screen->w % 2  == 0);
    assert(screen->h % 2  == 0);
    assert(bufsize >= (screen->w/2) * (screen->h/2));
    // Get pixel value for each
    Uint8 r, g, b;
    for(int y=0; y<screen->h; y += 2) {
        for(int x=0; x<screen->w; x += 2) {
            unsigned int gray_val = 0;
            SDL_GetRGB(getPixel(screen, x, y), screen->format, &r, &g, &b);
            gray_val += r + g + b;
            SDL_GetRGB(getPixel(screen, x+1, y), screen->format, &r, &g, &b);
            gray_val += r + g + b;
            SDL_GetRGB(getPixel(screen, x, y+1), screen->format, &r, &g, &b);
            gray_val += r + g + b;
            SDL_GetRGB(getPixel(screen, x+1, y+1), screen->format, &r, &g, &b);
            gray_val += r + g + b;
            *buf = gray_val / 12;
            ++buf;
        }
    }

    return buf - oldbuf;
}

int Server::makeScreenshotColor(Game::State *gs, char *buf, int bufsize) {
    // Returns size of buf

    char *oldbuf = buf;

    // Get the screen
    SDL_Surface *screen = General::copyToTempSurface(gs->getScreen());
    assert(screen != 0);
    assert(screen->w == 320);
    assert(screen->h == 240);
    assert(bufsize >= (screen->w/2) * (screen->h/2));
    // Get pixel value for each
    Uint8 r, g, b;
    for(int y=0; y<screen->h; y += 2) {
        for(int x=0; x<screen->w; x += 2) {
            unsigned int sum_r=0, sum_g=0, sum_b=0;
            SDL_GetRGB(getPixel(screen, x, y),     screen->format, &r, &g, &b);
            sum_r += r; sum_g += g; sum_b += b;
            SDL_GetRGB(getPixel(screen, x+1, y),   screen->format, &r, &g, &b);
            sum_r += r; sum_g += g; sum_b += b;
            SDL_GetRGB(getPixel(screen, x, y+1),   screen->format, &r, &g, &b);
            sum_r += r; sum_g += g; sum_b += b;
            SDL_GetRGB(getPixel(screen, x+1, y+1), screen->format, &r, &g, &b);
            sum_r += r; sum_g += g; sum_b += b;
            *buf = sum_r / 4;
            ++buf;
            *buf = sum_g / 4;
            ++buf;
            *buf = sum_b / 4;
            ++buf;
        }
    }

    return buf - oldbuf;
}


bool Server::pollSockets() {
    // Polls sockets and processes new connections/input
    // as is possible. Returns true if we found any activity.
    // Otherwise returns false.
    //

    // See how many sockets need stuff done to them
    int socks_ready = SDLNet_CheckSockets(_set, 1); // spend 1 msec

    // If none or error (eew), split.
    if(socks_ready <= 0)
        return false;

    // Otherwise let's see what we can do!

    // check listen socket
    if(SDLNet_SocketReady(_listen)) {
        TCPsocket new_sock = SDLNet_TCP_Accept(_listen);
        addConnection(new_sock);
        --socks_ready;
    }

    // check peers
    PeerData *pdata;
    for(unsigned int peerid=0;
            socks_ready>0 && peerid < _peers.size();
            peerid++ ) {

        // foreach peer, process data if we have any
        if(  (pdata = _peers[peerid])  &&
                SDLNet_SocketReady(pdata->tcpsock)  ) {

            if(!pdata->buf.readSock(pdata->tcpsock) || pdata->hasError()) {
                if (pdata->hasError()) { cout << "Disconnecting sock due to sock error.\n"; }
                
                // If we didn't get any data, trash the connection
                if (pdata->hasError())
                    delConnection(peerid, "Corresponding pdata had errors.");
                else 
                    delConnection(peerid, "Had trouble reading from socket.");
            } else {
                // ### HANDLE THE EVENT AND POLL AGAIN (CONSIDER 'WHILE')...
                // ### .. DON'T FORGET TO SET THE BUF SIZE TO 0 AGAIN!!!
                char * msg;
                while((msg = pdata->buf.getEvent())) {
                    // got a message!
                    if(debugProtocol)
                        cout << "C@" << pdata->peerid << " <" << setw(3) << SMSG_ELEN(msg) << ", " << setw(3) << SMSG_TYPE(msg) << ">: ";

                    switch (SMSG_TYPE(msg)) {
                    case MC_KEY_UP:
                        if (debugProtocol)
                            cout << "KEY_UP " << Uint16(*SMSG_DATA(msg)) << endl;

                        assert(pdata->player && pdata->player->isPlayer() && SMSG_ELEN(msg) == 1);
                        ((Player *) pdata->player)->setKeyState(*SMSG_DATA(msg), false);
                        break;

                    case MC_KEY_DOWN:
                        if (debugProtocol)
                            cout << "KEY_DOWN " << Uint16(*SMSG_DATA(msg)) << endl;

                        assert(pdata->player && pdata->player->isPlayer() && SMSG_ELEN(msg) == 1);
                        ((Player *) pdata->player)->setKeyState(*SMSG_DATA(msg), true);
                        break;

                    case MC_STATE_INIT_REMOTE:
                        // Client: "I want the game state to be updated over time."
                        //         "Also, send me the rest of the init info."
                        //
                        if (debugProtocol)
                            cout << "I want my gamestate to be updated over time.\n";

                        pdata->wants_game_state = true;

                        // DON'T BREAK -- continue below

                    case MC_STATE_INIT: {
                           if (debugProtocol)
                               cout << "Send me initialization info for current gamestate.\n";

                            // Send game state to non-local client
                            if (pdata->wants_game_state) {
                                _gs->sendInitGameState(pdata);
                            }

                            // This is who you are
                            Uint32 pid = hton32(pdata->player->getId());
                            
                            pdata->sendMsg(MS_SET_PID, (char *) &pid, sizeof(pid));

                            break;
                        }


                    case MC_REQUEST_SCREENSHOT: {
                           if (debugProtocol)
                               cout << "Send me a screenshot of the game.\n";

                            assert(pdata->player != 0);

                            char buf[100000];
                            cout << "MAKING\n";
                            int msgsize = makeScreenshot(pdata->player->getGameState(), buf, 100000);
                            cout << "Made screenshot\n";
                            pdata->sendMsg(MS_SCREENSHOT, buf, msgsize);

                            break;
                        }

                    case MC_SET_PLAYERNAME: {
                           if (debugProtocol)
                               cout << "Set playername to '" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << "'\n";

                            assert(pdata->player != 0);
                            
                            string oldname = pdata->player->getName();
                            
                            // only treat it special if the new name is different.
                            if (oldname != string(SMSG_DATA(msg), SMSG_ELEN(msg))) {
                                // Set player's name to whatever they asked
                                pdata->player->setNameUnique(string(SMSG_DATA(msg), SMSG_ELEN(msg)));

                                // do nothing special
                                if (oldname.length()) {
                                    _gs->appendConsoleMsgAll(oldname + " changed " + pdata->player->getGender().getHis() + " name to " + pdata->player->getName() + ".");
                                } else {
                                    _gs->appendConsoleMsgAll(pdata->player->getName() + " has entered the game.");
                                }
                            }
                            
                            break;
                        }
                    
                    case MC_SET_PLAYERGENDER: {
                        assert(SMSG_ELEN(msg) >= 1);
                        
                        if (debugProtocol)
                            cout << "Set gender to '" << string(SMSG_DATA(msg), 1) << "'\n";

                        assert(pdata->player != 0);
                        pdata->player->getGender().setGender(SMSG_DATA(msg));
                        
                        break;
                    }
                    
                    case MC_SAY_TEXT:{
                           if (debugProtocol)
                               cout << "Say text '" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << "'\n";
                               
                            assert(pdata->player != 0);
                            
                            _gs->appendConsoleMsgAll("<" + pdata->player->getName() + ">: " + string(SMSG_DATA(msg), SMSG_ELEN(msg)) );
                            
                            break;
                        }
                    
                    case MC_USE_UDP: {
                        assert(SMSG_ELEN(msg) == 2);
    // ########## UDP CRAZINESS ##############
                        Uint16 cli_udp_port = ntoh16( * ((Uint16*) SMSG_DATA(msg)) );
                        
                        if (debugProtocol)
                           cout << "I want to use UDP on my port " << cli_udp_port
                                << ". Hook me up!\n";

                        assert(pdata->wants_game_state == true);

                        IPaddress *tcpip = SDLNet_TCP_GetPeerAddress(pdata->tcpsock);
                        assert(tcpip != 0);

                        IPaddress udpip;
                        udpip.host = tcpip->host;
                        udpip.port = hton16(cli_udp_port);
                        
                        // Accept UDP'age from them
                        assert(pdata->udpchan == -1);
                        
                        if( (pdata->udpchan=SDLNet_UDP_Bind(_udpsock, -1, &udpip)) == -1) {
                            printf("SDLNet_UDP_Bind: %s\n", SDLNet_GetError());
                            exit(1);
                        }
                        _udp_user_count++;
//                        cout << "bound to channel " << pdata->udpchan << endl;
                        break;
                    }

                    case MC_DO_COMMAND: {
                           if (debugProtocol)
                               cout << "Do command '" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << "'\n";
                               
                            assert(pdata->player != 0);
                            pdata->player->doCmd(string(SMSG_DATA(msg), SMSG_ELEN(msg)));
                            
                            break;
                        }

                    default:
                        cout << "CLIENT SENT UNKNOWN MESSAGE: len " << SMSG_LEN(msg)
                        << ", type " << SMSG_TYPE(msg)
                        << ", elen " << SMSG_ELEN(msg) << endl;
//                        << ", msg [" << string(SMSG_DATA(msg), SMSG_ELEN(msg)) << ']'
//                        << endl;
                    };

                    pdata->buf.resetBuf();
                }
            }
        }

    }

    return true;
}

void Server::sendAllState(char *msg, int len, bool trivial) {
    // Used to send RAW state messages - doesn't send them to
    // those who don't want it.
    //
    
    assert(isMessageWellFormed(msg, len));
    
//        cout << "SENDING RAW MSG OF SIZE " << SMSG_LEN(msg) << ", TYPE "
//             << SMSG_TYPE(msg) << ", alleged len: " << len << endl;

    for(unsigned int i=0; i<_peers.size(); ++i) {
        if(_peers[i] && _peers[i]->wants_game_state && (!trivial || _peers[i]->udpchan == -1))
            _peers[i]->sendMsgRaw(msg, len);
    }

    // Send to leet UDP'ers
    if (_udp_user_count && trivial) {
        UDPpacket *packet = SDLNet_AllocPacket(1024);
        if(!packet) {
            printf("SDLNet_AllocPacket: %s\n", SDLNet_GetError());
            exit(1);
        } else {
            // do stuff with this new packet
            // SDLNet_FreePacket this packet when finished with it

            memcpy(packet->data, msg, len);
            packet->len = len;

            int numsent = SDLNet_UDP_Send(_udpsock, 0, packet);
//            cout << "NUMSENT: " << numsent << endl;
            if (!numsent) {
                // could fail if no addresses are bound to channel
                printf("SDLNet_UDP_Send failed (MTYPE %d, LEN %d): %s\n", SMSG_TYPE(msg), len, SDLNet_GetError());
            }
            SDLNet_FreePacket(packet);
        }
    }

    _bytes_mass_sent += len;
}


//
//
//  ########## EVENTS BELOW THIS POINT ##############
//
//


void Server::onConnect(PeerData *pdata) {
    assert(pdata != 0);

    // Let's see where they connected from!
    IPaddress *ip = SDLNet_TCP_GetPeerAddress(pdata->tcpsock);
    if (!ip) {
        printf("SDLNet_TCP_GetPeerAddress: %s\n", SDLNet_GetError());
        printf("This may be a server socket.\n");
        exit(1);
    }

    /*char *host;
    if(!(host=SDLNet_ResolveIP(ip))) {
        printf("SDLNet_ResolveIP: %s\n", SDLNet_GetError());
        exit(1);
    }*/
    cout << "###    CONNECTED: Peer " << pdata->peerid << " @ " << getIPString(ip) << endl;
    

    
    assert(_gs != 0);


    // WELCOME THE MOFO'S!!
    //

    // Send versioning info - This is important and should send first :-)
    char data[10];
    *((Uint16 *) data    ) = hton16(SALP_WARS_MIN_VERSION);
    pdata->sendMsg(MS_MIN_VERSION, data, 2);

    // Associate new player object with this socket
    Game::PlayerStomper *player = new Game::PlayerStomper();
    player->setPeerData(pdata);
    player->setMap(_gs->getStartMap());
    player->setPosStart();
    _gs->addItem(player);

    // associate with peerdata
    pdata->player = player;
    assert(pdata->player->isPlayer());
    
    // Add to scoreboard
    _gs->getScoreboard()->addPlayer(player->getId());
}

void Server::onDisconnect(PeerData *pdata) {
    IPaddress *ip = SDLNet_TCP_GetPeerAddress(pdata->tcpsock);
    if (!ip) {
        printf("SDLNet_TCP_GetPeerAddress: %s\n", SDLNet_GetError());
        printf("This may be a server socket.\n");
        exit(1);
    }
    cout << "### DISCONNECTED: Peer " << pdata->peerid << " @ " << getIPString(ip) << endl;



    _gs->appendConsoleMsgAll(pdata->player->getName() + " has exited the game.");


    if (pdata->udpsock)  {
        SDLNet_UDP_Unbind(pdata->udpsock, pdata->udpchan);
        --_udp_user_count;
    }

    _gs->delItem(pdata->player->getId());
    pdata->player = 0;
}


bool Server::sendSlotServerScreenshot(string host, string myport, string path, bool color)
{
    // Let's tell the slot server about us!
    // Returns true on success (well, not really, but pretty close).
    
    assert(_gs != 0);

    static HTTPData http_data;
    // NOTE: This is bad coding, really. If this function were called from
    //       multiple threads, there is a race condition below. Good thing
    //       this isn't possible yet. :)
    if (http_data.in_transit) {
        cout << "You are already trying to post this.\n";
        return false;
    }
    http_data.in_transit = true;
    
    string tmpbody;
    char picbuf[100000];
    int piclen = color ? makeScreenshotColor(_gs, picbuf, 100000) :
                         makeScreenshot(_gs, picbuf, 100000);
    string querystring = "what=screenshot&port=" + myport
                       + "&color_screenshot="    + (color?string("yes"):string("no"))
                       + "&" + _gs->getSlotServerDesc();
                       
    http_data.host = host;
    http_data.port = 80;
    http_data.path = path + "?" + querystring;
    http_data.post_data = string(picbuf, piclen);
    SDL_CreateThread(HTTPPostData, &http_data); // Threaded, so we out of here early :-).

    return true;
}

bool Server::sendSlotServerGameState(string host, string myport, string path)
{
    // Let's tell the slot server about us!
    // Returns true on success (well, not really, but pretty close).
    
    assert(_gs != 0);

    static HTTPData http_data;
    // NOTE: This is bad coding, really. If this function were called from
    //       multiple threads, there is a race condition below. Good thing
    //       this isn't possible yet. :)
    if (http_data.in_transit) {
        cout << "You are already trying to update the server with GameState info.\n";
        return false;
    }
    http_data.in_transit = true;
                       
    http_data.host = host;
    http_data.port = 80;
    http_data.path = path + "?port=" + myport
                       + "&" + _gs->getSlotServerDesc();
    
    SDL_Thread * t = SDL_CreateThread(HTTPGet, &http_data); // Threaded, so we out of here early :-).
    if (!t) {
        fprintf(stderr, "Error sending slot server gamestate (couldn't create thread): %s\n", SDL_GetError());
        http_data.in_transit = false;
    }
    
    return true;
}



} // namespace Game
} // namespace Salp
