#ifndef __GAMESERVER_H
#define __GAMESERVER_H

#include <vector>
#include <cassert>

#include "SALPSTUB.h"
#include "Sock.h"
#include "OSD.h"
#include "Game/State.h"

#define MAX_SOCKETS_SERVED 32
#define SERVER_PORT 12394

using namespace std;

namespace Salp {
namespace Game {


class Server
{
public:
    Server(Game::State *gs);
    ~Server();

    // Adds <new_sock> to list of peers, returning its peerid
    unsigned int addConnection(TCPsocket new_sock);
    void delConnection(unsigned int peerid, string reason);  // Disconnects peer <peerid>
    unsigned int getBytesMassSent();
    bool listenOn(int port = SERVER_PORT);
    
    static int makeScreenshot(Game::State *gs, char *buf, int bufsize);
    static int makeScreenshotColor(Game::State *gs, char *buf, int bufsize);
    void onConnect(PeerData *pdata);
    void onDisconnect(PeerData *pdata);
    bool pollSockets();  // Checks sockets for, and handles, new activity.
                         // Returns true if there was any socket activity.
    void sendAll(char *msg, int len);
    void sendAllState(char *msg, bool trivial=false) { sendAllState(msg, SMSG_LEN(msg), trivial); } // Note: DO NOT CALL THIS FUNCTION WHEN BUILDING MULTI-MESSAGE STRINGS TO SEND!
    void sendAllState(char *msg, int len, bool trivial=false);
    bool sendSlotServerGameState(string host, string myport, string path);
    bool sendSlotServerScreenshot(string host, string myport, string path, bool color);  

    friend class Salp::Game::State;
    static Game::Server *_server;
    
private:
    unsigned int        _bytes_mass_sent; // bytes massively sent -- to a theoretical all-game player (through sendAll or sendAllState). not multiplied by num of users sent to
    Game::State        *_gs;
    TCPsocket           _listen;
    vector<PeerData *>  _peers;
    SDLNet_SocketSet    _set;
    UDPsocket          _udpsock;
    int                _udp_user_count;
};

class Client
{
public:
    Client(Game::State *gs, Game::Player **gp);
    ~Client();

    // Connect (TCP) to server at <host:port>
    bool connectTo(string host, int port);

    void handleServerMessage(char *msg);
    void handleServerMessages(char *msg, int len);

    // Send <len> bytes found in <data>
    bool sendRawData(char * data, int len);

    // Check for activity
    bool pollSocket();

    // shouldn't really be public, but..:
    PeerData         * _peer;

private:
    SDLNet_SocketSet   _set; // to check for available data
    Game::State      * _gs;
    Game::Player     ** _mainplayer;
};


} // namespace Game
} // namespace Salp

#endif
