#ifndef __GAMESOCK_H
#define __GAMESOCK_H

#if defined(__WIN32__) || defined(WIN32)
    #include <winsock.h>
#else
     /* !WIN32 */
    #include <sys/time.h>
    #include <unistd.h>
    #include <fcntl.h>
    #include <netinet/in.h>
    #include <sys/socket.h>
    #include <netinet/tcp.h>
#endif

#include "SALP_SDL.h"
#include "SDL_net.h"

#include <vector>
#include <cassert>
#include <iostream>


#include "SALPSTUB.h"
#include "Item.h"
#include "General/general.h"

using namespace Salp::General;

#define SBUF_SIZE 200
#define SBUF_MAX_MSG_SIZE SBUF_SIZE/2-4

/*
  SMSG_LEN: Message length, total. Size of LEN, TYPE and DATA fields
  SMSG_ELEN: Effective message length. Size of DATA field.
  SMSG_TYPE: Type of message (unsigned int)
  SMSG_DATA: ***POINTER*** to DATA field
*/
#define SMSG_LEN(a)   ntoh16(*((Uint16 *) a))
#define SMSG_ELEN(a)  ( ntoh16(*((Uint16 *) a)) - 4)
#define SMSG_TYPE(a)  ntoh16(*((Uint16 *) (a + 2) ))
#define SMSG_DATA(a)  (a + 4)
// Note: "I" means "I will increment <data> afterwards!" :)
#define DATA_ICOPY_8(data, prop)  { assert(sizeof(prop) == 1);  *((char *) data) = *((char *) &prop); ++data; }
#define DATA_IREAD_8(data, prop)  { assert(sizeof(prop) == 1);  *((char *) &prop) = *((char *) data); ++data; }
#define DATA_ICOPY_16(data, prop)  {assert(sizeof(prop) == 2);  SDLNet_Write16((Uint16) prop, data); data += 2; }
#define DATA_IREAD_16(data, prop)  {assert(sizeof(prop) == 2);  prop = SDLNet_Read16(data); data += 2; }
#define DATA_ICOPY_32(data, prop)  {assert(sizeof(prop) == 4);  SDLNet_Write32((Uint32) prop, data); data += 4; }
#define DATA_IREAD_32(data, prop)  {assert(sizeof(prop) == 4);  prop = SDLNet_Read32(data); data += 4; }
#define DATA_ICOPY_N(data, propptr, len)  { memcpy(data, propptr, len); data += len; }
#define DATA_IREAD_N(data, propptr, len)  { memcpy(propptr, data, len); data += len; }
#define DATA_ICOPY_CSTR(data, cstr, len) { \
     *((Uint16 *) data) = hton16(len);            \
     data += 2;                           \
     strncpy(data, cstr, len);            \
     data += len;                         \
}
#define DATA_IREAD_STR(data, stlstr) { \
     stlstr = string(data + 2, ntoh16(*((Uint16 *) data)));   \
     data += 2 + *((Uint16 *) data);                  \
}
#define SET_SMSG_LEN(a, b)   *((Uint16 *) a) = hton16(b);
#define SET_SMSG_TYPE(a, b)  *((Uint16 *) (a + 2))  = hton16(b);
#define SET_SMSG_DATA_STR(a, b)  memcpy((void *) (a + 4), (void *) b.c_str(), b.length() + 1);

// Legacy SW network code
#define DATA_COPY_AND_INC(data, prop) { \
    if (sizeof(prop) == 1)      DATA_ICOPY_8(data, prop) \
    else if (sizeof(prop) == 2) DATA_ICOPY_16(data, prop) \
    else if (sizeof(prop) == 4) DATA_ICOPY_32(data, prop) \
    else  assert(0);  /* data type not supported! */ \
}

#define DATA_READ_AND_INC(data, prop) { \
    if (sizeof(prop) == 1)      DATA_IREAD_8(data, prop) \
    else if (sizeof(prop) == 2) DATA_IREAD_16(data, prop) \
    else if (sizeof(prop) == 4) DATA_IREAD_32(data, prop) \
    else  assert(0);  /* data type not supported! */ \
}


using namespace std;

namespace Salp {
namespace Game {

    enum GameMsgType { 
       MC_KEY_UP             =  1,   // Client: Player pressed key <keyid>
       MC_KEY_DOWN           =  2,   // Client: Player pressed key <keyid>
       MC_STATE_INIT         =  3,   // Client: Requesting Gamestate Initialization
       MC_STATE_INIT_REMOTE  =  4,   // Client: I am remote .. and requesting initial game state

       MS_LOAD_LEVEL         =  5,   // Server: Load level <level id>
       MS_SET_PID            =  6,   // Server: You are playerid <pid>
       MS_CREATE_ITEM        =  7,   // Server: create this item
       MS_UPDATE_ITEM        =  8,   // Server: update this item
       MS_DELETE_ITEMS       =  9,   // Server: delete these ITEMS (use SMSG_SIZE to figure out how many)
       MS_SOUND_PLAY_ONCE    = 10,   // Server: play sound <n> once.
       MS_MIN_VERSION        = 11,   // Server: You better be at least <min version>
       
       MC_REQUEST_SCREENSHOT = 12,    // Client: Give me a bitmap screenshot of the game!
       MS_SCREENSHOT         = 13,    // Server: Here's your damn screenshot!
       MS_CONSOLE_MESSAGE    = 14,    // Server: Display this message in the console window.
       MC_SET_PLAYERNAME     = 15,    // Client: My name is <str>. (ticka ticka slim shady)
       MC_DO_COMMAND         = 16,    // Client: Execute this command on the server-side.
       MC_SAY_TEXT            = 17,    // Client: Tell this message to everyone in the game.
       MC_SET_PLAYERGENDER   = 18,     // Client: my gender is <char>
       MC_USE_UDP            = 19,     // Client: I want to use UDP.. please! I'm listening on port <msg>
       MS_LOAD_TILE_SET      = 20,     // Server: Load tileset named <name> into gamestate
       MS_LOAD_SOUND         = 21,     // Server: Load sound named <name> into gamestate
       MS_UPDATE_PLAYER_SCORE = 22
    };

class SockBuff {
public:
    SockBuff() {
        _bufsize = 0;
    };
    
    bool readSock(TCPsocket sock);
    char * getEvent();
    void resetBuf() {
        _bufsize = 0;
    }

private:
    char          _buf[SBUF_SIZE + sizeof(Uint16)];
    unsigned int  _bufsize;
};

class PeerData {
public:
    PeerData();
    ~PeerData();

    // message type, data, message length
    bool hasError() { return _has_error; }
    int sendMsg(Uint16 mtype, const char *data, Uint16 len);
    void sendMsg(Uint16 mtype, string data);
    void sendMsg(const char * val, Uint16 len);    // not checked for full message packaging
    void sendMsgRaw(const char * val, Uint16 len); // checked for full message packaging
    void setError(string msg);
    
    SockBuff      buf;
    unsigned int  peerid;
    Game::Player   *player;
    TCPsocket     tcpsock;
    bool          wants_game_state;
    UDPsocket     udpsock;
    int           udpchan;
    Uint16        udpport;

private:

    bool          _has_error;

};

}
}

struct HTTPData {
    HTTPData() { in_transit = false; }
    string host;
    int    port;
    string path;
    string post_data;
    volatile bool in_transit;
};

bool HTTPGet(string host, int port, string path, string &body); 
int HTTPGet(void *http_data); // Takes HTTPData * as argument, for threading
bool HTTPPostData(string host, int port, string path, string &data, string &body);
int  HTTPPostData(void *http_data); // Takes HTTPData * as argument, for threading
string getIPString(IPaddress *ip);
#endif
