#include "general.h"

#ifdef WIN32
#include <windows.h>
#include <direct.h>
#else
#include <unistd.h>
#include <pwd.h>
#endif

#include <cstdio>
#include <cstring>
#include <strstream>
#include <errno.h>
#include <fstream>

#include "Game/Sock.h"

namespace Salp {
namespace General {

char tmp_data[1024];           // temporary data storage (largely for network message-building stuff)
bool debugScoreboard = false;
bool debugGraphics   = false;
bool debugProtocol   = false; // set to true to see details of received messages.
bool useTCPNoDelay   = false;
bool useTrainingMode = false; // sends extra warning messages



bool associateSalpProtocolWithApp()
{
    //used to error check, but now it doesn't since it's silly. we'll assume this is a normal, functioning copy of windows
#ifdef WIN32
    HKEY hKey;

    char editflags[4] = {2, 0, 0, 0};  // No clue, but OK! :)

    // Build quoted filepath string
    char path[512];
    path[0] = '\"';
	GetModuleFileName(NULL, path+1, 509); // copy my exe path to string
    strcat(path, "\"");     
    
    // Create key for salp protocol
    RegCreateKey(HKEY_CLASSES_ROOT, "salp", &hKey);
    RegSetValue(hKey, NULL, REG_SZ, "URL:salp Protocol", sizeof("URL:salp Protocol"));      //isnt' that sizeof trick cool?
    RegSetValueEx(hKey, "URL Protocol", 0, REG_SZ, (const BYTE *)"", 0);
    RegSetValueEx(hKey, "EditFlags", 0, REG_BINARY, (const BYTE *)editflags, 4);
    RegCloseKey(hKey);

    // Set up default icon for any linky url files made with the protocol (not necessary)
    RegCreateKey(HKEY_CLASSES_ROOT, "salp\\DefaultIcon", &hKey);
    RegSetValue(hKey, NULL, REG_SZ, path, strlen(path));
    RegCloseKey(hKey);

    
    //this will end up executing: "path" "url" add /url= or whatever if you need for the command line
    strcat(path, " \"%1\"");
    RegCreateKey(HKEY_CLASSES_ROOT, "salp\\shell\\open\\command", &hKey);
    RegSetValue(hKey, NULL, REG_SZ, path, strlen(path));
    RegCloseKey(hKey);

    return true;
#else
    return false;
#endif
}

string normalizeFPath(string fpath) {
    // changes /'s to \ on win32, and vice versa on lunix
    for (unsigned int i=0; i<fpath.length(); i++) {
        #ifdef WIN32
            if (fpath[i] == '\\')  fpath[i] = '/';
        #else
            if (fpath[i] == '/')  fpath[i] = '\\';
        #endif
    }

    return fpath;
}

bool downloadSalpFile(string fpath, unsigned int fsize) {
    // Downloads and replaces game files.. a keen way to auto-update
    // stuff if I say so myself. returns true on success, false on failure
    //

    string fdata;
    string local_fpath = normalizeFPath(getBaseSalpDir() + "\\" + fpath);
        
    cout << "    o  Downloading " << fpath << endl;
    if (!HTTPGet(gPrefBucket->getVal("SS_HOST"),
            80,
            gPrefBucket->getVal("SS_PATH_QUERY") + "?what=get_salp_file&fpath=" + URLEncode(fpath),
            fdata) || fdata.length() != fsize)
    {
        cout << "EXPECTED SIZE: " << fsize << "; actually " << fdata.length() << endl;
        cerr << "Salp Wars encountered an error when trying to download a file from the slot server. Please, don't cry! Try again later, or check the website (http://" << gPrefBucket->getVal("SS_HOST") << "/)" << endl;
        return false;
    }

    ofstream fout;
    fout.open(local_fpath.c_str(), ios::binary);
    if (fout.fail()) {
        cerr << "    x  Could not open file " << local_fpath << " for writing.\n";
        return false;
    }
    fout.write(fdata.c_str(), fdata.length());
    fout.close();
    cout << "    o  Saved (" << fdata.length() << " bytes)\n";
    return true;
}

void checkForAutoUpdates() {

    assert(gPrefBucket != 0);
    
    cout << "*** Checking with master server for game updates " << gPrefBucket->getVal("SS_PATH_QUERY") << "\n";
    
    // Download update list
    string allist;
    if (!HTTPGet(gPrefBucket->getVal("SS_HOST"),
            80,
            gPrefBucket->getVal("SS_PATH_QUERY") + "?what=get_autoupdate_list",
            allist) || allist.length() == 0)
    {
        cerr << "Salp Wars encountered an error when checking the slot server for game updates. Please, don't cry! Try again later, or check the website (http://" << gPrefBucket->getVal("SS_HOST") << "/)" << endl;
        exit(1);
    }
    
    // We've got the update list; traverse it 
    strstream ss;
    unsigned int fsize=0;
    string fname, fmodtime;
    
    ss << allist;
    
    Game::PrefBucket *alfiles = new Game::PrefBucket("prefs/autoload.ini");
    
    ss >> fsize >> fmodtime >> fname;
    
    while (!ss.fail()) {
        cout << "[" << fsize << "\t--\t" << fmodtime << "\t--\t" << fname << "]\n";
        if (alfiles->getVal(fname, "") != fmodtime) {
//            downloadSalpFile(fname, fsize);
            cout << "WOULD HAVE DOWNLOADED " << fname << endl;
            alfiles->setVal(fname, fmodtime);
        }
        
        ss >> fsize >> fmodtime >> fname;
    }
    
    alfiles->save();
    delete alfiles;

    cout << "*** Done checking for game updates.\n";
}


string getBaseSalpDir()
{
#ifdef WIN32
    string s = getDirPath(getEXEFilePath());
    assert(s.length() > 4);
    return s.substr(0, s.rfind("\\", s.length()-2));
#else
    assert(0);
    return "???";
#endif
}

string getEXEFilePath()
{
    // Returns path to the exe running; note that this is not
    // guaranteed to work. If it fails, returns empty string
    // ("").
    
    char path[1024];
#ifdef WIN32
    GetModuleFileName(NULL, path, 1024);
#else
    /*
       OOPS, this is cwd, not exe filepath
       
       if (!getcwd(path, 512)) {
        cout << "Error getting EXE filepath: " << strerror(errno) << endl;
        exit(1);
    }*/
    
    // Unix proggies shouldn't need to use this function; if they do, uh..
    // this could be a bit more work (involving searching with argv[0] and
    // the PATH (see http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC23 )
    
    // If the user has /proc/ we can get this!
    pid_t my_pid = getpid(); // getppid(); <-- would have returned /bin/bash
    char link_path[1024];
    sprintf(&link_path[0], "/proc/%d/exe", my_pid);
    
    int chars_written = readlink(&link_path[0], &path[0], 1024);
    if (chars_written > 0) {
        return string(&path[0], chars_written);
    } else { 
        cerr << "Warning: I could not figure out where this EXE is running\n"
             << "         from. This may cause the game to exit shortly\n"
             << "         after it is run (since it cannot find media files)\n";
        return "";
    }
    
#endif
    return path;
}

string getDirPath(string file_path) 
{
    // returns path to DIR of filepath
#ifdef WIN32
    return file_path.substr(0, file_path.rfind("\\"));
#else
    return file_path.substr(0, file_path.rfind("/"));
#endif

}

string getUserName()
{
#ifdef WIN32
    return "Taery";
#else
    struct passwd * uinfo = getpwuid(geteuid());
    return uinfo->pw_name;
#endif
}


bool setWorkingDirectory(const string &path)
{
    assert(path.length() > 0);
    
    // Returns true on success
    return chdir((char *) path.c_str()) == 0;
}

string URLEncode(string msg)
{
    string encmsg; // encoded message
    
    for (unsigned int i=0; i<msg.length(); ++i) {
        char c = msg[i];
        if (c >= 'a' && c <= 'z'   ||
            c >= 'A' && c <= 'Z'   ||
            c >= '0' && c <= '9')
        {
            encmsg += c;
        } else {
            char hexval[20];
            sprintf(hexval, "%x", c);
            encmsg += "%";
            encmsg += hexval;
        }
    }

    return encmsg;
}

static void itoa_buf(int i, char * s1)
{
    // Mich
    
    char c[20];

	if(i < 0)
		*s1++ = '-';

	char *s2 = c;

	do
	{
		*s2++ = '0' + (i % 10);
		i /= 10;
	} while(i != 0);

	s2--;
	for(; s2 >= c; *s1++ = *s2--);
	*s1++ = 0;
}

string itoa(int i)
{
	char c[20];
	itoa_buf(i, c);
	return c;
}

int sign (int n) {
    if (!n) return 0;
    return  n<0?-1:1;
}

void trim (char *s) {
    // Removes beginning and trailing whitespace from cstring s
    if (!*s) { return; } // no need to trim 0-length strings!

    char *readhead = s + strlen(s), *puthead = s;

    // Trim ENDING characters
    while (isspace(*(readhead-1)))    --readhead;
    *readhead = '\0';

    // Trim BEGINNING characters
    readhead = s;
    while (isspace(*readhead))  ++readhead; // Slide to first non-whitespace char
    if (readhead != puthead)
        do {
            *puthead = *readhead;
            ++puthead; ++readhead;
        } while (*readhead != '\0');
}

};
};
