#ifndef SALP_MEDIA_H
#define SALP_MEDIA_H

#include "SALPSTUB.h"
#include "Game/PrefBucket.h"
#include "General/general.h"
#include "General/graphics.h"
#include "Game/State.h"
#include "General/RandBucket.h"

#include <cassert>
#include <map>

#ifdef SALP_MEDIA_CXX
    #define DECLARE_MEDIA_VAR(type, name) type * name = 0;
    #define DECLARE_MEDIA_VAR_ARRAY(type, name, size) type * name[size];
    #define DECLARE_MEDIA_VAR_RAND(type, name) General::RandBucket<type> name;

    #define ADD_SOUND_VAR(name, deffname) \
        name.addItem(gGameState->getSound("sounds/" #deffname , true));

    #define INIT_SOUND_VAR(name, deffname) \
         name = gGameState->getSound(p.getVal(#name, "sounds/" #deffname ), true);  \
         assert(name != 0);

    #define ADD_SEQ_VAR(name, deffname) \
        name.addItem(gGameState->getTileSequence("tiles/" #deffname , true));

    #define INIT_SEQ_VAR(name, deffname) \
         name = gGameState->getTileSequence(p.getVal(#name, "tiles/" #deffname ), true);  \
         assert(name != 0);

#else
    #define DECLARE_MEDIA_VAR(type, name) extern type * name;
    #define DECLARE_MEDIA_VAR_ARRAY(type, name, size) extern type * name[size];
    #define DECLARE_MEDIA_VAR_RAND(type, name) extern General::RandBucket<type> name;
#endif

using namespace std;

// ----------------------------------------------------------------------

namespace Salp {
namespace General {

DECLARE_MEDIA_VAR(Game::Sound, sound_taer_i_am_a_big_tall_rat);
DECLARE_MEDIA_VAR(Game::Sound, sound_shotgun);
DECLARE_MEDIA_VAR(Game::Sound, sound_salp_splat);

DECLARE_MEDIA_VAR_RAND(Game::Sound, sound_taer_taunt);
DECLARE_MEDIA_VAR_RAND(Game::Sound, sound_taer_cheer);
DECLARE_MEDIA_VAR_RAND(Game::Sound, sound_taer_ouch);
DECLARE_MEDIA_VAR_RAND(Game::Sound, sound_taer_jump);

DECLARE_MEDIA_VAR(Tile::Sequence, seq_spreadshot_bullets);
DECLARE_MEDIA_VAR_ARRAY(Tile::Sequence, seq_shotgun_bullets, 9);
DECLARE_MEDIA_VAR_ARRAY(Game::Sound, sound_multi_kill, 4);
DECLARE_MEDIA_VAR_ARRAY(Tile::Sequence, seq_salp_vomit, 9);
DECLARE_MEDIA_VAR_ARRAY(Tile::Sequence, seq_salp_vomit_explosion, 9);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_salp_grape);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_salp_frog);
DECLARE_MEDIA_VAR_ARRAY(Tile::Sequence, seq_salp_grape_explosion, 9);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_powerup_salp_grape);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_powerup_salp_frog);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_powerup_shotgun);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_powerup_spreadshot);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_powerup_homing);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_powerup_taken);
DECLARE_MEDIA_VAR(Tile::Sequence, seq_powerup_health);

// ----------------------------------------------------------------------
void initStaticMedia();
#ifdef SALP_MEDIA_CXX

void initStaticMedia()
{
    Game::PrefBucket p("prefs/initmedia.ini");

    //    INIT_SOUND_VAR(sound_taer_i_am_a_big_tall_rat, taer_i_am_a_big_tall_rat.wav)
    //    INIT_SOUND_VAR(sound_taer_ouch, taer_ouch2.wav)

    INIT_SOUND_VAR(sound_shotgun, shotgun.wav)
    INIT_SOUND_VAR(sound_salp_splat, splat.wav)


    INIT_SEQ_VAR(seq_spreadshot_bullets, ammo_shotgun/9);

    sound_multi_kill[0] = gGameState->getSound("sounds/double_salp.wav", true);
    sound_multi_kill[1] = gGameState->getSound("sounds/triple_salp.wav", true);
    sound_multi_kill[2] = gGameState->getSound("sounds/super_salp.wav", true);
    sound_multi_kill[3] = gGameState->getSound("sounds/salpy_salp.wav", true);

    // 0 1 2
    // 3 4 5   <--- firing directions, indices into seq_shotgun_bullets
    // 6 7 8
    seq_shotgun_bullets[0] = gGameState->getTileSequence("tiles/ammo_shotgun/1", true);
    seq_shotgun_bullets[1] = gGameState->getTileSequence("tiles/ammo_shotgun/2", true);
    seq_shotgun_bullets[2] = gGameState->getTileSequence("tiles/ammo_shotgun/3", true);
    seq_shotgun_bullets[3] = gGameState->getTileSequence("tiles/ammo_shotgun/4", true);
    seq_shotgun_bullets[4] = 0;
    seq_shotgun_bullets[5] = gGameState->getTileSequence("tiles/ammo_shotgun/5", true);
    seq_shotgun_bullets[6] = gGameState->getTileSequence("tiles/ammo_shotgun/6", true);
    seq_shotgun_bullets[7] = gGameState->getTileSequence("tiles/ammo_shotgun/7", true);
    seq_shotgun_bullets[8] = gGameState->getTileSequence("tiles/ammo_shotgun/8", true);
    assert(seq_shotgun_bullets[0] != 0);

    // 0 1 2
    // 3 4 5   <--- firing directions, indices into seq_salp_vomit
    // 6 7 8
    seq_salp_vomit[0] = gGameState->getTileSequence("tiles/salp/1", true);
    seq_salp_vomit[1] = gGameState->getTileSequence("tiles/salp/2", true);
    seq_salp_vomit[2] = gGameState->getTileSequence("tiles/salp/3", true);
    seq_salp_vomit[3] = gGameState->getTileSequence("tiles/salp/4", true);
    seq_salp_vomit[4] = 0;
    seq_salp_vomit[5] = gGameState->getTileSequence("tiles/salp/5", true);
    seq_salp_vomit[6] = gGameState->getTileSequence("tiles/salp/6", true);
    seq_salp_vomit[7] = gGameState->getTileSequence("tiles/salp/7", true);
    seq_salp_vomit[8] = gGameState->getTileSequence("tiles/salp/8", true);
    assert(seq_salp_vomit[0] != 0);

    // 0 1 2
    // 3 4 5   <--- firing directions, indices into seq_salp_vomit
    // 6 7 8
    seq_salp_vomit_explosion[0] = gGameState->getTileSequence("tiles/salp/9", true);
    seq_salp_vomit_explosion[1] = gGameState->getTileSequence("tiles/salp/10", true);
    seq_salp_vomit_explosion[2] = gGameState->getTileSequence("tiles/salp/11", true);
    seq_salp_vomit_explosion[3] = gGameState->getTileSequence("tiles/salp/12", true);
    seq_salp_vomit_explosion[4] = 0;
    seq_salp_vomit_explosion[5] = gGameState->getTileSequence("tiles/salp/13", true);
    seq_salp_vomit_explosion[6] = gGameState->getTileSequence("tiles/salp/14", true);
    seq_salp_vomit_explosion[7] = gGameState->getTileSequence("tiles/salp/15", true);
    seq_salp_vomit_explosion[8] = gGameState->getTileSequence("tiles/salp/16", true);
    assert(seq_salp_vomit_explosion[0] != 0);

    // 0 1 2
    // 3 4 5   <--- firing directions, indices into seq_shotgun_bullets
    // 6 7 8
    seq_salp_grape_explosion[0] = gGameState->getTileSequence("tiles/grape/1", true);
    seq_salp_grape_explosion[1] = gGameState->getTileSequence("tiles/grape/2", true);
    seq_salp_grape_explosion[2] = gGameState->getTileSequence("tiles/grape/3", true);
    seq_salp_grape_explosion[3] = gGameState->getTileSequence("tiles/grape/4", true);
    seq_salp_grape_explosion[4] = 0;
    seq_salp_grape_explosion[5] = gGameState->getTileSequence("tiles/grape/5", true);
    seq_salp_grape_explosion[6] = gGameState->getTileSequence("tiles/grape/6", true);
    seq_salp_grape_explosion[7] = gGameState->getTileSequence("tiles/grape/7", true);
    seq_salp_grape_explosion[8] = gGameState->getTileSequence("tiles/grape/8", true);
    assert(seq_salp_grape_explosion[0] != 0);

    INIT_SEQ_VAR(seq_salp_grape, grape/9);
    INIT_SEQ_VAR(seq_salp_frog, frog/1);
    INIT_SEQ_VAR(seq_powerup_shotgun, powerups/2);
    INIT_SEQ_VAR(seq_powerup_spreadshot, powerups/3);
    INIT_SEQ_VAR(seq_powerup_salp_frog, powerups/4);
    INIT_SEQ_VAR(seq_powerup_salp_grape, powerups/5);
    INIT_SEQ_VAR(seq_powerup_homing, powerups/6);
    INIT_SEQ_VAR(seq_powerup_taken, powerups/7);
    INIT_SEQ_VAR(seq_powerup_health, powerups/8);

    ADD_SOUND_VAR(sound_taer_cheer, taer_yeah.wav)
    ADD_SOUND_VAR(sound_taer_cheer, taer_yeah2.wav)
    ADD_SOUND_VAR(sound_taer_cheer, taer_yeah3.wav)
    ADD_SOUND_VAR(sound_taer_cheer, taer_woohoo.wav)

    ADD_SOUND_VAR(sound_taer_taunt, taer_ha.wav)
    ADD_SOUND_VAR(sound_taer_taunt, taer_i_will_jump_higher.wav)
    ADD_SOUND_VAR(sound_taer_taunt, taer_i_am_a_big_tall_rat.wav)
    ADD_SOUND_VAR(sound_taer_taunt, taer_i_just_make_the_noises.wav)
    ADD_SOUND_VAR(sound_taer_taunt, taer_whats_up_my_pal.wav)

    ADD_SOUND_VAR(sound_taer_ouch, taer_ouch.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_ouch2.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_aah.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_aah2.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_aah3.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_aah4.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_aah5.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_aah6.wav)
    ADD_SOUND_VAR(sound_taer_ouch, taer_aah7.wav)

    ADD_SOUND_VAR(sound_taer_jump, taer_jump.wav)
    ADD_SOUND_VAR(sound_taer_jump, taer_jump2.wav)
    ADD_SOUND_VAR(sound_taer_jump, taer_jump3.wav)
    ADD_SOUND_VAR(sound_taer_jump, taer_jump4.wav)
    ADD_SOUND_VAR(sound_taer_jump, taer_jump5.wav)
    ADD_SOUND_VAR(sound_taer_jump, taer_jump6.wav)


    p.save();
}
#endif


};
};

#endif
