#include "input.h"
#include <fstream>
#include <iostream>
#include "SALPSTUB.h"

#include "SDL_mixer.h"

using namespace Salp;

/* XPM */
const char *arrow[] = {
    /* width height num_colors chars_per_pixel */
    "    32    32        5            1",
    /* colors */
    "X c #000000",
    ". c #ffffff",
    "s c #660000",
    "w c #000066",
    "  c None",
    /* pixels */
    "X                               ",
    "XX                              ",
    "X.X                             ",
    "X..X                            ",
    "X...XXXXXXXXXXXXXXXXXXXXXXXXX   ",
    "X...........................X   ",
    "X...........XXXXXX.....X....X   ",
    "X..........X.....X..X..X....X   ",
    "X...........XX...X..X..X....X   ",
    "X.............X...X.X.X.....X   ",
    "X.....XXXXXXXX.....X.X......X   ",
    "X...........................X   ",
    "X.XXX..XXXXXXXXXXXXXXXXXXXXXX   ",
    "XX  X..X                        ",
    "X    X..X                       ",
    "     X..X                       ",
    "      X..X                      ",
    "      X..X                      ",
    "       XX                       ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "                                ",
    "0,0"
};

SDL_Cursor *initSystemCursor(const char *image[]) {
    int i, row, col;
    Uint8 data[4*32];
    Uint8 mask[4*32];
    int hot_x, hot_y;

    i = -1;
    for ( row=0; row<32; ++row ) {
        for ( col=0; col<32; ++col ) {
            if ( col % 8 ) {
                data[i] <<= 1;
                mask[i] <<= 1;
            } else {
                ++i;
                data[i] = mask[i] = 0;
            }
            switch (image[4+row][col]) {
            case 'X':
                data[i] |= 0x01;
                mask[i] |= 0x01;
                break;
            case '.':
                mask[i] |= 0x01;
                break;
            case ' ':
                break;
            }
        }
    }
    sscanf(image[4+row], "%d,%d", &hot_x, &hot_y);
    return SDL_CreateCursor(data, mask, 32, 32, hot_x, hot_y);
}

//
// Keyboard Map Code (save/load/default)
//
//
bool saveKeyboardMap(map<int, char>& keyMap, string fpath) {
    //
    // Saves keyboard map data to file at <fpath>,
    // overwriting all data currently in the file.
    //

    // Open keyboard map file
    ofstream fout;
    fout.open(fpath.c_str(), ios::out );
    if(!fout.is_open())
        return false;

    char currline[200];

    // Cycle through keyboard map data, and save it!
    map<int, char>::iterator i;
    for(i=keyMap.begin(); i != keyMap.end(); i++) {
        sprintf(currline, "%x %x", i->first, i->second);
        fout << "KEYMAP_BYCODE " << currline << endl;
    }
    
    fout.close();

    return true;
}

bool loadKeyboardMap(map<int, char>& keyMap, string fpath) {
    // Attempts to load the keyboard map file at <fpath>. If failure,
    // returns false.

    // Open keyboard map file
    ifstream fin;
    fin.open(fpath.c_str());


    // Can't open the file for reading? Bummer!
    if(!fin.is_open()) {
        cout << "Couldn't open keyboard map \"" << fpath << "\" for reading.\n";
        return false;
    }

    unsigned int lineno = 0;
    char currline[200];
    char tokenstr[20], tempstr[200];
    while(!fin.eof() && fin.good()) {
        // get the next line of the file
        ++lineno;
        fin.getline(currline, 200-1);

        // skip comments
        if(sscanf(currline, " # %s", tempstr) ||
           !strlen(currline))
            continue;

        // otherwise get token name
        if(!sscanf(currline, " %20s %[^\r]", tokenstr, tempstr)) {
            cout << "loadKeyboardMap: Unrecognized Line " << lineno << ": " << currline << endl;
        } else if (tokenstr == string("KEYMAP_BYCODE")) {
            // Keyboard mapping by [hex] code.
            unsigned int keyCode=0, gameCode=0;
            if(!sscanf(tempstr, " %x %x ", &keyCode, &gameCode))
                cout << "(" << lineno << ") Could not parse keyCode.\n";
            else
                keyMap[keyCode] = gameCode;
        } else {
            // Didn't recognize the token? (Oh no!)
            cout << "(" << lineno << ") Unrecognized Token: "
            << tokenstr << endl;
        }
    }

    if(!fin.good() && !fin.eof()) {
        cout << "An error occurred when reading the level file, at line "
        << (lineno)
        << "." <<endl;
        exit(1);
    }


    return true;
}

void loadKeyboardMapDefaults(map<int, char>& keyMap) {
    keyMap['i'] = Game::KEY_UP;
    keyMap['j'] = Game::KEY_LEFT;
    keyMap['k'] = Game::KEY_DOWN;
    keyMap['l'] = Game::KEY_RIGHT;

    keyMap[SDLK_KP8] = Game::KEY_UP;
    keyMap[SDLK_KP4] = Game::KEY_LEFT;
    keyMap[SDLK_KP2] = Game::KEY_DOWN;
    keyMap[SDLK_KP6] = Game::KEY_RIGHT;
    keyMap[SDLK_KP0] = Game::KEY_L;

    keyMap[SDLK_UP] = Game::KEY_UP;
    keyMap[SDLK_LEFT] = Game::KEY_LEFT;
    keyMap[SDLK_DOWN] = Game::KEY_DOWN;
    keyMap[SDLK_RIGHT] = Game::KEY_RIGHT;


    keyMap[SDLK_SPACE] = Game::KEY_Y;
    keyMap['z']   = Game::KEY_R;
    keyMap['r']   = Game::KEY_R;
    //    keyMap[SDLK_SHIFT]   = Game::KEY_B;

    keyMap['s'] = Game::KEY_L;
    keyMap['d'] = Game::KEY_B;
    keyMap['a'] = Game::KEY_B;

    keyMap['e'] = Game::KEY_X;
    keyMap['w'] = Game::KEY_A;
}

void loadJoystickMapDefaults(map<int, char>& joyMap, string joyname) {
    // Guillemot
    if (joyname == "Guillemot Corporation DA Leader") {
        joyMap[4] = Game::KEY_Y; // LEFT
        joyMap[7] = Game::KEY_B; // BOTTOM
        joyMap[6] = Game::KEY_A; // RIGHT
        joyMap[5] = Game::KEY_X; // TOP
        joyMap[0] = Game::KEY_L; // FRONT LEFT
        joyMap[2] = Game::KEY_R; // FRONT RIGHT
        joyMap[1] = Game::KEY_L; // BACK  LEFT
        joyMap[3] = Game::KEY_R; // BACK  RIGHT

        joyMap[10] = Game::KEY_L; // BACK  LEFT - trigger
        joyMap[11] = Game::KEY_R; // BACK  RIGHT - trigger
        // joyMap[12] = Game::KEY_R; // CENTER, between two analog
    }
    else if (joyname == "Logitech Inc. WingMan Gamepad") {
        // Logitech WingMan Gamepad
        joyMap[1] = Game::KEY_B; // B
        joyMap[2] = Game::KEY_A; // C
        joyMap[4] = Game::KEY_Y; // Y
        joyMap[5] = Game::KEY_X; // Z
        joyMap[6] = Game::KEY_L; // L
        joyMap[7] = Game::KEY_R; // R

        joyMap[0] = Game::KEY_L; // A
        joyMap[3] = Game::KEY_R; // X
        joyMap[8] = Game::KEY_R; // START
        joyMap[9] = Game::KEY_R; // MACRO
        joyMap[10] = Game::KEY_R; // SELECT
    }
    else if (joyname == "Gravis GamePad Pro USB ") {
        // Gamepad Pro
        joyMap[0] = Game::KEY_Y; // RED
        joyMap[1] = Game::KEY_B; // YELLOW
        joyMap[2] = Game::KEY_A; // GREEN
        joyMap[3] = Game::KEY_X; // BLUE
        joyMap[4] = Game::KEY_L; // FRONT LEFT
        joyMap[5] = Game::KEY_R; // FRONT RIGHT
        joyMap[6] = Game::KEY_L; // BACK  LEFT
        joyMap[7] = Game::KEY_R; // BACK  RIGHT
    }
    else {
        cout << "### WARNING\n"
             << "### Your joystick has not been tested with Salp Wars\n"
             << "### and hence your default joystick settings may not work as\n"
             << "### expected. Feel free to customize the joystick.ini manually.\n";

        // Generic (??)
        joyMap[0] = Game::KEY_Y; // RED
        joyMap[1] = Game::KEY_B; // YELLOW
        joyMap[2] = Game::KEY_A; // GREEN
        joyMap[3] = Game::KEY_X; // BLUE
        joyMap[4] = Game::KEY_L; // FRONT LEFT
        joyMap[5] = Game::KEY_R; // FRONT RIGHT
        joyMap[6] = Game::KEY_L; // BACK  LEFT
        joyMap[7] = Game::KEY_R; // BACK  RIGHT
    }
}

int getJoyButton() {
    // Waits for a gamepad BUTTON to be pressed, and returns that
    // button's jbutton id. If the ESCape key has been pressed,
    // returns -1
    //

    SDL_Event event;

    while ( 1 ) {
        SDL_PollEvent(&event);
        switch (event.type) {
        case SDL_KEYDOWN:
            if(event.key.keysym.sym == SDLK_ESCAPE)
                return -1;
            break;
        case SDL_JOYBUTTONDOWN:
            return event.jbutton.button;
            break;
        }

        SDL_Delay(100); // Don't hog CPU
    }

}

bool mapJoyButtonInteractively(map<int, char>& joyMap, string promptText, int gamebutton) {
    cout << promptText << "\n";

    int butt;
    if((butt = getJoyButton()) >= 0 ) {
        joyMap[butt] = gamebutton;
        return true;
    } else
        return false;
}

bool configJoystickInteractively(map<int, char>& joyMap, string joyname) {
    cout << "We are going to interactively configure your joystick settings.\n"
    << "In order to play the game, you need to specify the following keys.\n"
    << "Press the button on your gamepad as I list each of them off.\n\n"
    << "Make sure you're using joystick '" << joyname << "'\n\n";

    if( mapJoyButtonInteractively(joyMap, "   o   A-button ( right-most of 4 on SNES) - toggles weapon", Game::KEY_A) &&
            mapJoyButtonInteractively(joyMap, "   o   B-button (bottom-most of 4 on SNES) - jumps", Game::KEY_B) &&
            mapJoyButtonInteractively(joyMap, "   o   Y-button (  left-most of 4 on SNES) - primary attack", Game::KEY_Y) &&
            mapJoyButtonInteractively(joyMap, "   o   X-button (   top-most of 4 on SNES) - secondary attack", Game::KEY_X) &&
            mapJoyButtonInteractively(joyMap, "   o   L-button (at top-left      on SNES) - hold to run", Game::KEY_L) &&
            mapJoyButtonInteractively(joyMap, "   o   R-button (at top-right     on SNES) - used for taunts??", Game::KEY_R)
      ) {

        cout << "\nNeat, we're set.\n";
        return true;
    } else {
        cout << "Okay, we'll stop configuration. No saving here!\n";
        return false;
    }
}

//
// Audio Code
//
//
Mix_Music *music = NULL;

/* This is the function that we told SDL_Mixer to call when the music
   was finished. In our case, we're going to simply unload the music
   as though the player wanted it stopped.  In other applications, a
   different music file might be loaded and played. */
void musicDone() {
    Mix_HaltMusic();
    Mix_FreeMusic(music);
    music = NULL;
}

