#ifndef __SALPSTUB_H
#define __SALPSTUB_H

// Version, multiplied by 100 (always integer)
#define SALP_WARS_VERSION 44

// If you change the protocol in a major way (or at all)
// then the following version should match the above version:
#define SALP_WARS_MIN_VERSION 41

#define MAX_TEAMS 4

namespace Salp {
    namespace Tile {
        class Map;
        class Sequence;
        class Set;
        class Loader;
    }
    namespace Game {

        enum WeaponType {
           WEAPON_UNKNOWN=0,         // default

           WEAPON_HOMING,         // primary: homing;             secondary: tracer
           WEAPON_SALP,           // primary: salp grape/frog;      secondary: vomit (weaker spread salp?)
           WEAPON_SPREAD,
           WEAPON_SHOTGUN,        // primary: fire shot           secondary: 

           WEAPON_MINE,           // primary: throw mine forward  secondary: drop mine
           WEAPON_PROXY_MINE,     // primary: fire proxy mine
           WEAPON_REMOTE_MINE,    // primary: fire mine           secondary: explode mineS
           WEAPON_MACHINEGUN,     // primary: rapid-fire shot     secondary:  
           WEAPON_NET,            // primary: fast, slows down targ   secondary: slow, freezes player
           WEAPON_DUMMY_NPC,      // primary: 
           WEAPON_BLOCK,          // primary: drops static brick block below the player (and player moves up above block)
           WEAPON_REMOTE_MISSILE,
           WEAPON_BOXING_GLOVE,   // primary: hurts, but doesn't fly far;   secondary: sends victim flying, doesnt hurt
           WEAPON_BOOMERANG,
           WEAPON_ROCKET, 

           // The following are secondary weapons, etc
           WEAPON_TRACER
        };

        enum PowerUpType {
           POWERUP_SPEED_DEC,    // <amt>
           POWERUP_SPEED_INC,    // <amt>
           POWERUP_IMMOBILIZE,   // <sec>
           POWERUP_SHRINK,       // getting back is the problem.. make them suicide!
           POWERUP_ANTI_GRAVITY, // jeff says "use balloon hat"
           POWERUP_TEAM_NEUTRAL  // paints player gray or something, so people dont know what team he's on
        };

        // This is used to index into the Game::Player _keys[] array.
        // the _keys[] array values are true when the player
        // has those buttons down.
        // **** DO NOT CHANGE WHAT THE EXISTING VARS VALUES ARE!! *****
        const unsigned char KEY_UP=0, KEY_DOWN=1, KEY_LEFT=2, KEY_RIGHT=3,
               KEY_A=4, KEY_B=5, KEY_X=6, KEY_Y=7, 
               KEY_L=8, KEY_R=9;

        // Values from getType()
        enum { GI_CAN_MOVE                = 1 << 0,    // i.e. is animatePos called?
               GI_CAN_COLLIDE_WITH_MAP    = 1 << 1,    // should I check item-map collisions?
               GI_CAN_COLLIDE_WITH_ITEM   = 1 << 2,    // should I check item-item collisions?
               GI_IS_VISIBLE              = 1 << 3,    // should I bother drawing the image?
               GI_CAN_ANIMATE             = 1 << 4,    // should I bother animate()ing the obj (aside from position)?
               GI_NOT_DELETED             = 1 << 5,
               GI_USE_VEL_THRESH          = 1 << 6,    // UNUSED
               GI_ON_COLLIDE_LOSE_VEL     = 1 << 7,
               GI_ON_COLLIDE_TAKE_VEL     = 1 << 8,
               GI_IS_AFFECTED_BY_GRAVITY  = 1 << 9,
               GI_IS_AFFECTED_BY_TRACTION = 1 << 10,
               GI_ON_COLLIDE_GIVE_VEL     = 1 << 11,
               GI_CAN_RIDE_ITEMS          = 1 << 12,
               GI_ON_DISCON_TAKE_VEL      = 1 << 13,
               GI_ON_COLLIDE_REPOS        = 1 << 14,
               GI_FACING_LEFT             = 1 << 15,
               GI_FACING_RIGHT            = 1 << 16,
               GI_FACING_UP               = 1 << 17,
               GI_FACING_DOWN             = 1 << 18,
               GI_USE_LOOSE_OOB          = 1 << 19
             };

        enum { DIRTY_SEQUENCE             = 1 << 0,    // Dirty bits
               DIRTY_XPOS                 = 1 << 1,
               DIRTY_YPOS                 = 1 << 2,
               DIRTY_MAP                  = 1 << 3,
               DIRTY_TYPE                 = 1 << 4,
               DIRTY_HP                   = 1 << 5,
               DIRTY_XVEL                 = 1 << 6,
               DIRTY_YVEL                 = 1 << 7,
               DIRTY_NAME                 = 1 << 8    // NPCs only
             };

        enum { GI_FORCED_LEFT=1,
               GI_FORCED_DOWN=2,
               GI_FORCED_UP=4,
               GI_FORCED_RIGHT=8,
               GI_FORCED_ANYWHERE=8 | 4 | 2 | 1
             };
        enum { DIR_LEFT=1,
               DIR_DOWN=2,
               DIR_UP=4,
               DIR_RIGHT=8,
               DIR_ANYWHERE=8 | 4 | 2 | 1
             };

        enum { 
               ITYPE_GENERIC=0, ITYPE_NPC=1, ITYPE_PLAYER=2, ITYPE_EFFECT=3
             };

        class Client;
        class ConvBubble;
        class Event;
        class Item;
        class ItemEffect;
        class ItemProjectile;
        class NPC;
        class OSD;
        class PeerData;
        class Player;
        class PlayerStomper;
        class Scoreboard;
        class Server;
        class State;
        class SockBuff;
        class Sound;
        class Vector;
        class StartLoc;
    }
};

#endif
