#include <iostream>
#include <assert.h>
#include "Tile/Sequence.h"
#include "General/graphics.h"

using namespace Salp::General;

namespace Salp {
namespace Tile {

Sequence::Sequence(Tile::Set *tileset, unsigned short framecount)
{
    _tileset = tileset;
    _framecount = 0;
    _frames = new TileFrame[framecount]; // used to be +1000; why?
    _cycle_time = 0;

    if(_frames == 0) {
        cerr << "Could not allocate memory for array of tile frames.\n";
        exit(1);
    }    
}

Sequence::~Sequence()
{
    delete[] _frames;

}

void Sequence::addFrame(unsigned short tileid, unsigned short framedelay)
{
    _frames[_framecount].tileid = tileid;
    _frames[_framecount].framedelay = framedelay;
    _cycle_time += framedelay;
    
    _framecount++;

}

void Sequence::drawImage(SDL_Surface *screen,
                         unsigned int& frameid,
                         const Sint32 x,
                         const Sint32 y,
                         unsigned int& frametimer,
                         const bool flipped) {
              
    assert(frameid < _framecount); // make sure the frame id exists :)
    
    if (flipped) {
        _tileset->drawImageHFlipped(screen, _frames[frameid].tileid, x, y);
    } else {
        _tileset->drawImage(screen, _frames[frameid].tileid, x, y);
    }
    
    // If the time has run out for this frame, advance to
    // the next frame.
    if (_framecount && SDL_GetTicks() - frametimer >= _frames[frameid].framedelay)
    {
        frametimer = SDL_GetTicks();
        frameid++;
        frameid %= _framecount;
    }    
}

void Sequence::drawRect(SDL_Surface *screen, const Sint32 x, const Sint32 y)
{
    static SDL_Rect dst;
    dst.h = _tileset->getHeight();
    dst.w = _tileset->getWidth();
    dst.x = x;
    dst.y = y;

    fillRectAlpha(screen, &dst,
                 SDL_MapRGB(screen->format,
                            0,  // R 
                            255,              // G
                            0         // B
                ), 
                   128/2 // half there, faster
                );
}

unsigned short Sequence::getFrameCount() const {
    return _framecount;
}

unsigned int Sequence::getHeight() const
{
    return _tileset->getHeight();
}

unsigned short Sequence::getId() const {
assert(0);
    return _seqid;
}

string Sequence::getName() const
{
    return _name;
}

unsigned int Sequence::getWidth() const
{
    return _tileset->getWidth();
}

void Sequence::setName(string name)
{
    _name = name;
}


} // namespace Tile
} // namespace Salp

