#ifndef __Sequence_H
#define __Sequence_H

#include <string>
#include "Tile/Set.h"
#include "Game/Item.h"

namespace Salp {
namespace Tile {


class TileFrame {
    public:
        unsigned short tileid, framedelay;
};

// A tile sequence is essentially a list of frames
class Sequence {
    public:
        explicit Sequence(Tile::Set *tileset, unsigned short framecount);
        ~Sequence();
        void addFrame(unsigned short tileid, unsigned short framedelay);
        void drawImage(SDL_Surface *screen,
                       unsigned int& frameid,
                       const Sint32 x,
                       const Sint32 y,
                       unsigned int& frametimer,
                       const bool flipped=false);
        void drawRect(SDL_Surface *screen, const Sint32 x, const Sint32 y);
        unsigned int getCycleTime() const { return _cycle_time; }
        unsigned short getFrameCount() const;
        unsigned int getHeight() const;
        unsigned short getId() const;
        string getName() const;
        unsigned int getWidth() const;
        void setId(const unsigned short seqid){ _seqid = seqid; }
        void setName(string name);
        
        friend class Salp::Game::Item;
    private:
        TileFrame *_frames;
        unsigned short _framecount;
        unsigned short _seqid;
        Tile::Set *_tileset;
        string _name;
        unsigned int _cycle_time; // time to go through one cycle
};


} // namespace Tile
} // namespace Salp

#endif
