#ifndef __Set_H
#define __Set_H

#include "SALP_SDL.h"
#include <string>

using namespace std;

namespace Salp {
namespace Tile {


class Set {
    public:
        explicit Set();
        ~Set();
        
        void drawImage(SDL_Surface *screen, unsigned short tileid, int x, int y, int alpha=0);
        void drawImageHFlipped(SDL_Surface *screen, unsigned short tileid, int x, int y, int alpha=0);

        string getName() const;
        SDL_Surface * getSimilarSurface(Uint32 width, Uint32 height);
        SDL_Surface * getSurface() { return _image; } 
        void changeColor(Uint32 cfrom, Uint32 cto);
        int getCount() const;
        int getHeight() const;
        int getWidth() const;
        Uint32 getTransparentColor() const;
        
        void loadImage(string bmpfname, SDL_Surface *screen=0);
        void matchPalette(SDL_Surface *screen);
        void setName(string name);
        void setCount(int tilecount);
        void setHeight(int tileh);
        void setWidth(int tilew);
        void setTransparentColor(unsigned char r, unsigned char g,
                unsigned char b);
        
        void verifyIntegrity();
        
    private:
//        void createHorizontallyFlipped();
        Uint32 getPixel(unsigned short x, unsigned short y);
    
        // The following are initialized when the Set
        // object is first created:
        //
        SDL_Surface *_image;          // tile's (BMP) image pointer
        int _tilew, _tileh; // # of pixels per frame of object
        int _tilecount;   // number of frames in image
        string _name;
        bool _hflipped; // true if image size has been doubled to horiz mirror
        Uint32 _transparentcolor; // transparent color of the image
        bool _hastransparentcolor; // if one is set, lemme know!
};

//
// Inline functions must be included here in order to be
// properly linked.
//



inline int Set::getHeight() const 
{
    return _tileh;
}

inline int Set::getWidth() const 
{
    return _tilew;
}


} // namespace Tile
} // namespace Salp


#endif
