#include <math.h>
#include <signal.h>
#include <ctype.h>
#include <vector>
#include <map>
#include <fstream>
#include <iostream>
#include <strstream>
#include <iomanip>
#include <csignal>
#include <stdlib.h>
#include <assert.h>
#include <cstdlib>
#ifndef WIN32
#include <unistd.h>
#include <sys/types.h>
#endif

#include "Game/Sock.h"
#include "Game/ItemPlatform.h"
#include "Game/ItemProjectile.h"
#include "Game/NPCBushBomber.h"
#include "Game/PathCP.h"
#include "Game/PathElliptical.h"
#include "Game/PlayerStomper.h"
#include "Game/Server.h"
#include "Game/Sound.h"
#include "General/general.h"
#include "General/graphics.h"
#include "General/media.h"
#include "SALPSTUB.h"
#include "SALP_SDL.h"

#include "Main/init.h"
#include "Main/input.h"

// *******************************************************
//     Constants
// *******************************************************

using namespace Salp;
using namespace Salp::General;

// *******************************************************
//     Globals
// *******************************************************

int           TICK_INTERVAL   = 0;//** SET BY COMMAND-LINE **; count on at least 10ms granularity, maybe coarser
bool          gCmdLineMode    = false;
bool          gAnimatedScreenshotMode = false;
bool          gDrawScoreSummary = false;
int          gAnimatedScreenshotID = 0;
bool          gIsIconified    = false;
Game::Client *gClient         = 0; // STATE DELETES THIS
Game::OSD    *gOSD            = 0; // STATE DELETES THIS
Game::PrefBucket *gPrefBucket = 0; // STATE DELETES THIS
Game::Player *gPlayer         = 0;
Game::PrefBucket gCmdLine;
Game::Server *gServer         = 0;
Game::Scoreboard *gScoreboard = 0;
Game::State  *gGameState      = 0;
int gFullScreen               = 0;
int gVersion  = SALP_WARS_VERSION; // div 100
int joyIndex                  = 0;
Sint32      gScreenX=0, gScreenY=0;
map<int, char> gJoystickMap; // key -> KEY_ constant
map<int, char> gKeyboardMap; // key -> KEY_ constant
SDL_Joystick *gJoystick       = 0;
SDL_Surface  *screen          = 0;
Uint16 gLastMouseX=0, gLastMouseY=0;

// *******************************************************
//     Prototypes
// *******************************************************

void animate();
int filterEvents(const SDL_Event *event);
void drawFPS();
void drawScreen();
void pollEvents();
Uint32 timeLeft();

void cleanup() {
    delete gGameState;
    gGameState = 0;
}

void onInterrupt(int signum) {
    exit(0);
}

int main(int argc, char *argv[]) {
    atexit(cleanup);

#ifndef WIN32
    signal(SIGINT,  onInterrupt);
    signal(SIGPIPE, SIG_IGN);
#endif

    // Display licensing stuffs
    cout << "\n----------------------------------------------------------------\n";
    printf("Salp Wars version %02d.%02da, Copyright (C) 2001, 2002 Tyler Goen\n", gVersion/100, gVersion % 100);
    cout << "Salp Wars comes with ABSOLUTELY NO WARRANTY.\n"
    << "This is free software, and you are welcome to redistribute it\n"
    << "under certain conditions. See 'LICENSE' and 'COPYING' for details.\n"
    << "----------------------------------------------------------------\n\n";

    // Set CWD to the dir that the executable's in.
    string exe_dir = General::getDirPath(General::getEXEFilePath());
    if (exe_dir.length() > 0)
        General::setWorkingDirectory(exe_dir);

    initCommandLine(argc, argv);

#ifndef WIN32
    if (geteuid() == 0) {
        cerr << "### You should not be running this program in superuser mode.\n";
        if (gCmdLine.getVal("safemode") == "yes") {
            cerr << "### Try again when you're not root!\n";
            exit(1);
        } else {
            cerr << "### Running it (as with may programs) could unintentionally\n"
                 << "### compromise your system.\n";
        }
    }
#endif

    initSDL();

    TICK_INTERVAL = gCmdLine.getIntVal("frame_delay");

    if (gCmdLine.getVal("screen") != "no") {
        initDisplay();
        initTempSurface(screen);
    }

    Game::Sound::enableAudio(gCmdLine.getVal("audio") == "yes");
    
    SDL_SetEventFilter(filterEvents);  
    initJoysticks();

    // ###########################################################
    // #   SPLASH SCREEN IS LIVE -- take your time.              #
    // ###########################################################

    if (gFullScreen) SDL_ShowCursor(0);        // Hide the cursor in full screen mode
    
    SDL_Cursor *custom_cursor = initSystemCursor(arrow);
    if(custom_cursor)
        SDL_SetCursor(custom_cursor);

    // ********************************************
    //  Load / Default and Save Preferences
    // ********************************************

    if (!loadKeyboardMap(gKeyboardMap, "prefs/keyboard.ini"))
        loadKeyboardMapDefaults(gKeyboardMap);

    saveKeyboardMap(gKeyboardMap, "prefs/keyboard.ini" );


    // ********************************************
    //  Initialize Game State
    // ********************************************

    gGameState = new Game::State();
    gScoreboard = gGameState->getScoreboard();
    Game::Item::setGameState(gGameState);
    gGameState->setScreen(screen);   // note that this may actually be zero!

    initPrefs();

#ifdef WIN32
    if (gCmdLine.getVal("autoupdate") == "yes") {
        checkForAutoUpdates();
    }
#endif

    initOSD(); // Note: this actually will load some media stuff into the gamestate

    gGameState->setOSD(gOSD);

    // Initialize [Network] server (if desired)
    // (note: "mode" could be "server" or "serveronly"; both require the init)
    if(gCmdLine.getVal("mode").substr(0,6) == "server")
        initServer();

    // Initialize [Network] client; that is, connect to the server
    if (gCmdLine.getVal("mode") != "serveronly")
        initClient();
   
    //
    // General networking stuffs
    //
#ifdef SIGPIPE
    signal(SIGPIPE, SIG_IGN); // Brass Pipes ;-) (Ignore pipe errors)
#endif

    // Keep doing sockety stuff if I'm a server, and I have activity
    while(gServer && gServer->pollSockets());
    while(gClient && gClient->pollSocket());
    while(gServer && gServer->pollSockets());
    while(gClient && gClient->pollSocket());

// This isn't accurate    gGameState->lockAutoLoad(); // Everything after this should be loaded explicitly.

    initMusic();
    
    // **
    // ** Main loop
    // **

    static unsigned int last_slot_server_update = 0,
                        last_slot_server_check  = 0;
               
    while(1) {
        pollEvents();
        while (gServer && gServer->pollSockets());
        while (gClient && gClient->pollSocket());
        while (gServer && gServer->pollSockets());

        static Uint8 *keys = SDL_GetKeyState(NULL);
        if(keys[SDLK_ESCAPE] == SDL_PRESSED)
            exit(0);
        timeLeft();
        animate();
        if (screen) drawScreen();

        if (gServer && ((SDL_GetTicks() - last_slot_server_check) > 60000 || !last_slot_server_check) && SDL_GetTicks() > 10000) {
            last_slot_server_check = SDL_GetTicks(); // dont try again for another 
            unsigned int ui = gPrefBucket->getIntVal("SS_UPDATEINTERVAL");
            if (ui && (SDL_GetTicks() - last_slot_server_update > 60000 * ui || !last_slot_server_update)) {
                last_slot_server_update = SDL_GetTicks();
                if (screen && gPrefBucket->getVal("SS_UPDATETYPE") == "color") {
                    gServer->sendSlotServerScreenshot(gPrefBucket->getVal("SS_HOST"), gCmdLine.getVal("port"), gPrefBucket->getVal("SS_PATH_REGISTER"), true);
                } else if (screen && gPrefBucket->getVal("SS_UPDATETYPE") == "grayscale") {
                    gServer->sendSlotServerScreenshot(gPrefBucket->getVal("SS_HOST"), gCmdLine.getVal("port"), gPrefBucket->getVal("SS_PATH_REGISTER"), false);
                } else {
                    gServer->sendSlotServerGameState(gPrefBucket->getVal("SS_HOST"), gCmdLine.getVal("port"), gPrefBucket->getVal("SS_PATH_REGISTER"));
                }
            }
        } 

        if(TICK_INTERVAL)
            SDL_Delay(timeLeft()); // not really accurate -- but who cares right now anyway?
    }

    /* This should never happen */
    printf("SDL_WaitEvent error: %s\n", SDL_GetError());

    return(0);
}


inline void drawFPS() {
    static unsigned int framecount=0;
    static Uint32 starttime=SDL_GetTicks();
    static char ctmp[100];

    sprintf(&ctmp[0], "%6.01f",  ++framecount * 1000.0 / (SDL_GetTicks() - starttime));
    Game::OSD::drawText5x5(screen, 5, screen->h-7, &ctmp[0], 6);
    
//    assert(0); // draw me!
//    gFnt->PutString(screen, 0, 0, ctmp);
//    gFnt->PutString(screen, 70, 0, "fps");
}

inline void drawNetStat() {
    if(!gServer)
        return;

    static Uint32 starttime=SDL_GetTicks();
    static char ctmp[100];

    sprintf(ctmp, "%6.01f",  (gServer->getBytesMassSent() * 1000.0 / (SDL_GetTicks() - starttime))); //* 1000.0 / SDL_GetTicks() - starttime);
    assert(0); // draw me!
//    gFnt->PutString(screen, 0, 20, ctmp);
//    gFnt->PutString(screen, 70, 20, "b/sec/user theoretical");
}

inline void animate() {
    // Animate.
    
    static Uint32 lastanimate = SDL_GetTicks();
    unsigned int acycles = (SDL_GetTicks() - lastanimate)*40/1000;
    lastanimate += acycles * 1000/ 40;

    if (acycles != 0) {
        if (gServer) {
             gGameState->animate(acycles);
        } else if (gPlayer) {
    //        gPlayer->setTypeOff(Game::GI_CAN_COLLIDE_WITH_MAP & Game::GI_CAN_COLLIDE_WITH_ITEM);
    //        gPlayer->animatePos(acycles);
        
            for (unsigned int i=acycles; i--; )
                gGameState->animatePosSoft(0); //gPlayer);
            gGameState->animateMaps(acycles);
        }
    }
    
    gOSD->animate(); // probably unnecessary in no-screen mode, but hey.
}

inline void drawScreen() {
    assert(screen != 0);
       
    // Draw Map (and items) from player's view
    // .. once we have a player established.
    if(gPlayer && !gIsIconified) {
        static Sint32 oldxpos=0, oldypos=0;

        if (oldxpos != gPlayer->getXPos() || oldypos != gPlayer->getYPos()) {

            gGameState->updateCameraPosition(screen, gPlayer, gScreenX, gScreenY, .25, .75);
            
//            cout << gPlayer->getXPos() << ", " << gPlayer->getYPos() << endl;
            gPlayer->getPosTL(oldxpos, oldypos);
        } else {
            /*
            Sint32 realScreenX = gScreenX, realScreenY = gScreenY;
            gGameState->updateCameraPosition(screen, gPlayer, realScreenX, realScreenY, 3, .66);
            */
            
            Sint32 realScreenX = gPlayer->getXPos() - (screen->w - gPlayer->getWidth())/2;
            Sint32 realScreenY = gPlayer->getYPos() - (screen->h - gPlayer->getHeight())/2;

            int maxscrollx   = (int) gPlayer->getMap()->getPixWidth()  - screen->w;
            int maxscrolly   = (int) gPlayer->getMap()->getPixHeight() - screen->h;
            
            Uint32 xdiff = abs(realScreenX - gScreenX);
            Uint32 ydiff = abs(realScreenY - gScreenY);
            
            if      (xdiff < 40) gScreenX += sign(realScreenX - gScreenX);
            else                 gScreenX += sign(realScreenX - gScreenX)*2;

            if      (ydiff < 40) gScreenY += sign(realScreenY - gScreenY);
            else                 gScreenY += sign(realScreenY - gScreenY)*2;

            if(gScreenY < 0)                gScreenY = 0;
            else if(gScreenY > maxscrolly)  gScreenY = maxscrolly;

            if(gScreenX < 0)                gScreenX = 0;
            else if(gScreenX > maxscrollx)  gScreenX = maxscrollx;
            
//            gScreenX = realScreenX; gScreenY = realScreenY;
        }
        
        gGameState->display(screen, gPlayer, gScreenX, gScreenY);
    }
    
    // Draw FPS readout
    drawFPS();
//    drawNetStat(); // .. network stats

    gOSD->drawImage(screen);

    if (gDrawScoreSummary)
        gScoreboard->drawScoreSummary(screen);

    if (gAnimatedScreenshotMode) {
        strstream ss;
        ss << "anim/screenshot" << setfill('0') << setw(3) << (gAnimatedScreenshotID++) << ".ppm";
        string tmp;
        ss >> tmp;
        savePPM(screen, tmp.c_str());
    }

    SDL_Flip(screen);
    return;
}

/* This function may run in a separate event thread */
int filterEvents(const SDL_Event *event) {
    //
    //  Returns 1 if the event should be handled, or 0 if
    //  the event should be dropped.
    //

    switch(event->type) {
        case SDL_MOUSEMOTION:
            
            if (gFullScreen && screen &&
                    ( event->motion.x != screen->w/2 ||
                      event->motion.y != screen->h/2 ) ) 
            {
                gScreenX += event->motion.xrel;
                gScreenY += event->motion.yrel;
                SDL_WarpMouse(screen->w/2, screen->h/2);
                return(0);   // drop event
            } 
            
            break;
            
    }

    return(1); // pass event on
}


inline void pollEvents() {
    SDL_Event event;

    /* Loop waiting for ESC+Mouse_Button */
    static int jval;

    while ( SDL_PollEvent(&event) ) {
        switch (event.type) {
        case SDL_ACTIVEEVENT: {
                if ( event.active.state & SDL_APPACTIVE ) {
                    if ( event.active.gain ) {
                        printf("App activated\n");
                        gIsIconified = false;
                    } else {
                        printf("App iconified\n");
                        gIsIconified = true;
                    }
                }
            }
            break;

        case SDL_KEYDOWN:  {
            SDLKey sym = event.key.keysym.sym;
            
            if(gCmdLineMode) {
                if (!(gCmdLineMode = gOSD->appendCmdLine(event.key.keysym.unicode & 0xFF))) {
                    SDL_EnableKeyRepeat(0, 0); // disable key repeat
                    SDL_EnableUNICODE(0);      // disable unicode translation
                }
            } else if (gClient && gKeyboardMap.find(sym) !=  gKeyboardMap.end())  {
                gClient->_peer->sendMsg(Game::MC_KEY_DOWN, &gKeyboardMap[sym], 1);
//                if (!gServer && sym >= Game::KEY_UP && sym <= Game::KEY_RIGHT)
//                    gPlayer->setKeyState(gKeyboardMap[sym], false);
            } else if (sym == 't') {
                gCmdLineMode = true;
                gOSD->setCmdLine("say ");
                SDL_EnableKeyRepeat(100, 60); // enable key repeat:  delay, interval
                SDL_EnableUNICODE(1);         // enable unicode translation
            } else if (sym == 'c') {
                gCmdLineMode = true;
                gOSD->setCmdLine("");
                SDL_EnableKeyRepeat(70, 40); // enable key repeat:  delay, interval
                SDL_EnableUNICODE(1);         // enable unicode translation
            }
            
            switch(sym) {
                case SDLK_F1:
                    if(SDL_WM_ToggleFullScreen(screen)) {
                        gFullScreen = !gFullScreen;
                        if (gFullScreen) SDL_ShowCursor(SDL_DISABLE); 
                        else             SDL_ShowCursor(SDL_ENABLE);
                    } else
                        cout << "##  Could not toggle fullscreenedness :( Sorry!\n##  This does not currently work on Win32; try running with 'screen=full'\n";
                    break;
                case SDLK_TAB:
                    gDrawScoreSummary = !gDrawScoreSummary;
                    //gOSD->appendMsg("Game score summary is now " +
                    //                (gDrawScoreSummary?string("shown"):string("hidden")) + ".");
                    break;
                case SDLK_F2:
                    gOSD->setShowRadar(!gOSD->getShowRadar());
                    //gOSD->appendMsg("Game radar is now " +
                    //                (gOSD->getShowRadar()?string("shown"):string("hidden")) + ".");
                    break;
/*                case SDLK_F2:
                    if(gServer) 
                        gServer->sendSlotServerScreenshot(gPrefBucket->getVal("SS_HOST"), gCmdLine.getVal("port"), gPrefBucket->getVal("SS_PATH_REGISTER"), false); // returns 0 on failure
                    break;
*/
                case SDLK_F3:
                    if(gServer) {
                        gServer->sendSlotServerScreenshot(gPrefBucket->getVal("SS_HOST"), gCmdLine.getVal("port"), gPrefBucket->getVal("SS_PATH_REGISTER"), true); // returns 0 on failure
                        gOSD->appendMsg("Attempting to send metaserver a pretty screenshot.");
                    } else
                        gOSD->appendMsg("Only game servers can send the metaserver screenshots.");
                    break;
                case SDLK_F4:
                    gPrefBucket->save(); 
                    gOSD->appendMsg("Saved preferences to disk.");
                    break;
                case SDLK_F5:
                    General::debugProtocol = !General::debugProtocol;
                    gOSD->appendMsg("Socket/Protocol debug mode now " +
                                    (General::debugProtocol?string("on"):string("off")) + ".");
                    break;
                case SDLK_F6:
                    General::useTCPNoDelay = !General::useTCPNoDelay;
                    gOSD->appendMsg("TCP_NODELAY " +
                                    (General::useTCPNoDelay?string("on"):string("off")) + " for NEW connections.");
                    break;
                case SDLK_F7:
                    General::debugGraphics = !General::debugGraphics;
                    gOSD->appendMsg("Graphic debug mode now " +
                                    (General::debugGraphics?string("on"):string("off")) + ".");
                    break;
                case SDLK_F8:
                    General::debugGraphics = !General::debugScoreboard;
                    gOSD->appendMsg("Scoreboard debug mode now " +
                                    (General::debugGraphics?string("on"):string("off")) + ".");
                    break;
                case SDLK_F9:
                    gAnimatedScreenshotMode = !gAnimatedScreenshotMode;
                    gAnimatedScreenshotID = 0;
                    cout << "Animated screenshot mode now " <<
                                    (gAnimatedScreenshotMode?string("on"):string("off")) << ".\n";
                    break;
                    
                case SDLK_PLUS:
                case SDLK_KP_PLUS:
                case SDLK_EQUALS:
                    if (!gCmdLineMode) {
                        Game::Sound::incMasterVolume(10);
                        gOSD->appendMsg("Volume now at " + Game::Sound::getMasterVolumeString() + "%");
                    }
                    break;
                case SDLK_MINUS:
                case SDLK_KP_MINUS:
                    if (!gCmdLineMode) {
                        Game::Sound::decMasterVolume(10);
                        gOSD->appendMsg("Volume now at " + Game::Sound::getMasterVolumeString() + "%");
                    }
                    break;

                default:
                    break; // do nothing        
            };
            
            break;
        }
            
        case SDL_KEYUP: {
                unsigned int sym = event.key.keysym.sym;
                if(!gCmdLineMode && gClient && gKeyboardMap.find(sym) != gKeyboardMap.end()) {
                    gClient->_peer->sendMsg(Game::MC_KEY_UP, &gKeyboardMap[sym], 1);
//                    if (!gServer && sym >= Game::KEY_UP && sym <= Game::KEY_RIGHT)
//                        gPlayer->setKeyState(gKeyboardMap[sym], false);
                }

                break;
            }
        case SDL_JOYAXISMOTION:
            jval = event.jaxis.value;
            //             cout << "Axis " << int(event.jaxis.axis) << ", JVAL " << jval << endl;
            if (gClient)
                if(event.jaxis.axis == 0) {
                    gClient->_peer->sendMsg(jval < -10000 ? Game::MC_KEY_DOWN : Game::MC_KEY_UP, (char *) &Game::KEY_LEFT, 1);
                    gClient->_peer->sendMsg(jval > 10000  ? Game::MC_KEY_DOWN : Game::MC_KEY_UP, (char *) &Game::KEY_RIGHT, 1);
                } else if (event.jaxis.axis == 1) {
                    gClient->_peer->sendMsg(jval < -10000 ? Game::MC_KEY_DOWN : Game::MC_KEY_UP, (char *) &Game::KEY_UP, 1);
                    gClient->_peer->sendMsg(jval > 10000  ? Game::MC_KEY_DOWN : Game::MC_KEY_UP, (char *) &Game::KEY_DOWN, 1);
                }
            break;

        case SDL_JOYBUTTONDOWN:
            //             cout << "BUTTDOWN " << int(event.jbutton.button) << endl;
            if(gClient && gJoystickMap.find(event.jbutton.button) != gJoystickMap.end())
                gClient->_peer->sendMsg(Game::MC_KEY_DOWN, &gJoystickMap[event.jbutton.button], 1);
            break;

        case SDL_JOYBUTTONUP:
            //              cout << "BUTTUP " << int(event.jbutton.button) << endl;
            if(gClient && gJoystickMap.find(event.jbutton.button) !=
                    gJoystickMap.end())
                gClient->_peer->sendMsg(Game::MC_KEY_UP, &gJoystickMap[event.jbutton.button], 1);
            break;

        case SDL_QUIT: {
                printf("Quit requested, quitting.\n");
                exit(0);
            }
            break;

        }

    }

}

inline Uint32 timeLeft() {
    // Returns the number of ms left until the next
    // iteration of our game loop.
    static Uint32 next_time = 0;
    Uint32 now = SDL_GetTicks();
    if ( next_time <= now ) {
        if(!next_time) {
            next_time = now+TICK_INTERVAL;
        } else
            next_time += TICK_INTERVAL;
        return(0);
    }

    return(next_time-now);
}
